// -*-c++-*-
#ifndef _RCC_AUDIO_H_
#define _RCC_AUDIO_H_

#include "rccplaymode.h"
#include "iostream.h"
#include "rccangle.h"
#include "rccgarbage.h"
#include "rcccoachmsg.h"
#include "rccside.h"
#include <iostream.h>
#include "rccsditem.h"

class RCC_Audio : public RCC_GCable, public RCC_SD_Item
{
public:
  RCC_Audio ( const int& t ) : time_ ( t ) {}
  virtual ~RCC_Audio ();
    
  virtual ostream& print ( ostream& o ) const = 0;

  friend ostream& operator<< ( ostream& o, const RCC_Audio& a ) { return a.print ( o ); }

protected:
  int time_;
};


class RCC_RefAudio : public RCC_Audio
{
public:
  RCC_RefAudio ( const int& t, const RCC_PlayMode& pm ) : RCC_Audio ( t ), play_mode ( pm ) {}
  RCC_RefAudio ( const int& t, const RCC_PlayMode& pm, const int& p ) : RCC_Audio ( t ), play_mode ( pm ) 
    {
	play_mode.points ( p );
    }

  virtual ~RCC_RefAudio ();

  virtual ostream& print ( ostream& o ) const;
  ostream& print ( ostream& o, const Side& ) const { return print ( o ); } 

  RCC_SD_Item* copy () const { return ( RCC_SD_Item* ) new RCC_RefAudio ( *this ); }
protected:    
  RCC_PlayMode play_mode;
};


class RCC_SelfAudio : public RCC_Audio
{
public:
  RCC_SelfAudio ( const int& t, const string& m ) : RCC_Audio ( t ), msg_ ( m ) {}

  virtual ~RCC_SelfAudio ();

  virtual ostream& print ( ostream& o ) const;
  ostream& print ( ostream& o, const Side& ) const { return print ( o ); } 

  RCC_SD_Item* copy () const { return ( RCC_SD_Item* ) new RCC_SelfAudio ( *this ); }

protected:   
  string msg_;
};


class RCC_PlayerAudio : public RCC_SelfAudio
{
public:
  RCC_PlayerAudio ( const int& t, const RCC_Angle& d, const string& m ) : RCC_SelfAudio ( t, m ), dir_ ( d ) {}

  virtual ~RCC_PlayerAudio ();

  virtual ostream& print ( ostream& o ) const;
  ostream& print ( ostream& o, const Side& ) const { return print ( o ); } 

  RCC_SD_Item* copy () const { return ( RCC_SD_Item* ) new RCC_PlayerAudio ( *this ); }

protected:   
  RCC_Angle dir_;
};


class RCC_CoachAudio : public RCC_Audio
{
public:
  RCC_CoachAudio ( const int& t, const RCC_Side& s, const string& m ) : RCC_Audio ( t ), side_ ( s ), time_sent ( t ) 
  {
    msg_p = new RCC_CoachFreeformMsg ( m );
  }

  RCC_CoachAudio ( const int& t, const RCC_Side& s, const int &ts, const RCC_CoachStdMsg* m ) 
    : RCC_Audio ( t ), side_ ( s ), time_sent ( ts ), msg_p ( m->copy () ) {}

  RCC_CoachAudio ( const RCC_CoachAudio& ca )
    : RCC_Audio ( ca ), side_ ( ca.side_ ), time_sent ( ca.time_sent ), msg_p ( ca.msg_p->copy () ) {}

  virtual ~RCC_CoachAudio ();

  virtual ostream& print ( ostream& o ) const;
  ostream& print ( ostream& o, const Side& ) const; 

  RCC_SD_Item* copy () const { return ( RCC_SD_Item* ) new RCC_CoachAudio ( *this ); }

protected:   
  RCC_Side side_;
  int time_sent;
  RCC_CoachStdMsg* msg_p;
};

#endif
