/* -*- Mode: C++ -*- */
/*
 *Copyright:

    Copyright (C) 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Itsuki Noda,	Mikhail Prokopenko, Jan Wendler 

    This file is part of SoccerServer.

    SoccerServer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

#ifndef _RCC_COACH_MSG_H_
#define _RCC_COACH_MSG_H_

#include <stream.h>
#include <vector>
#include <string>

#include "rccgarbage.h"
#include "param.h"
#include "rcccoachtoken.h"

/********* CoachStdMsg *******************/

class RCC_CoachStdMsg : public RCC_GCable
{
public:
  RCC_CoachStdMsg() {}
  virtual ~RCC_CoachStdMsg() {}

  virtual ostream& print(ostream& out) const = 0;
  virtual ostream& printPretty(ostream& out, const string& line_header) const = 0;

  friend ostream& operator <<(ostream & os, RCC_CoachStdMsg& m)
    { return m.print(os); }

  virtual RCC_CoachStdMsg* copy () const = 0;
} ;




class RCC_CoachMetaMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachMetaMsg () {}
  RCC_CoachMetaMsg ( const RCC_CoachMetaToken_List& l ) : tokens_ ( l ) {}
  
  ~RCC_CoachMetaMsg() {}

  ostream& print ( ostream& out) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  const RCC_CoachMetaToken_List& tokens () const { return tokens_; }
  void tokens ( const RCC_CoachMetaToken_List& l ) { tokens_ = l; }

  RCC_CoachStdMsg* copy () const { return ( RCC_CoachStdMsg* ) new RCC_CoachMetaMsg ( *this ); }

protected:
  RCC_CoachMetaToken_List tokens_;
} ;


/******* Freeform Message *********************/

class RCC_CoachFreeformMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachFreeformMsg ( string s = "" ) : str_ ( s ) {}
  ~RCC_CoachFreeformMsg () {}

  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;
    
  string str () const { return str_; }
  string str ( const string& s ) { return str_ = s; }

  RCC_CoachStdMsg* copy () const { return ( RCC_CoachStdMsg* ) new RCC_CoachFreeformMsg ( *this ); }
 
protected:
  string str_;
} ;



/******** Info Message ********************/
class RCC_CoachInfoMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachInfoMsg() {}
  RCC_CoachInfoMsg ( const RCC_CoachInfoToken_List& l ) : tokens_ ( l ) {}
  ~RCC_CoachInfoMsg() {}

  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  const RCC_CoachInfoToken_List& tokens () const { return tokens_; }
  void tokens ( const RCC_CoachInfoToken_List& l ) { tokens_ = l; }

  RCC_CoachStdMsg* copy () const { return ( RCC_CoachStdMsg* ) new RCC_CoachInfoMsg ( *this ); }

protected:
  RCC_CoachInfoToken_List tokens_;
};

/******** Advice Message ********************/
class RCC_CoachAdviceMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachAdviceMsg() {}
  RCC_CoachAdviceMsg ( const RCC_CoachInfoToken_List& l ) : tokens_ ( l ) {}
  ~RCC_CoachAdviceMsg() {}

  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  const RCC_CoachInfoToken_List& tokens () const { return tokens_; }
  void tokens ( const RCC_CoachInfoToken_List& l ) { tokens_ = l; }

  RCC_CoachStdMsg* copy () const { return ( RCC_CoachStdMsg* ) new RCC_CoachAdviceMsg ( *this ); }

protected:
  RCC_CoachInfoToken_List tokens_;
};


/******** Define Message ********************/
class RCC_CoachDefineMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachDefineMsg () {}
  RCC_CoachDefineMsg ( const RCC_CoachDefineToken_List& l ) : tokens_ ( l ) {}
  virtual ~RCC_CoachDefineMsg () {}

  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;
  
  RCC_CoachDefineToken_List& tokens() { return tokens_; }
  void tokens ( const RCC_CoachDefineToken_List& l ) { tokens_ = l; }

  RCC_CoachStdMsg* copy () const { return ( RCC_CoachStdMsg* ) new RCC_CoachDefineMsg ( *this ); }
 
protected:
  RCC_CoachDefineToken_List tokens_;
};


#endif 
