/* -*- Mode: C++ -*- */
/*
 *Copyright:

    Copyright (C) 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Itsuki Noda,	Mikhail Prokopenko, Jan Wendler 

    This file is part of SoccerServer.

    SoccerServer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

#ifndef _COACH_TOKEN_H_
#define _COACH_TOKEN_H_

#include <stream.h>
#include <vector>
#include <string>

#include "rccregion.h"
#include "rccgarbage.h"
#include "param.h"
#include "rccaction.h"
#include "rcccond.h"
#include "rccdirective.h"

class RCC_CoachToken : public RCC_GCable
{
public:
  RCC_CoachToken() {}
  virtual ~RCC_CoachToken() {}

  virtual ostream& print ( ostream& out ) const = 0;
  virtual ostream& printPretty ( ostream& out, const string& line_header) const = 0;
  friend ostream& operator << ( ostream& os, RCC_CoachToken& t)
    { return t.print(os); } 

  //  virtual RCC_CoachToken* copy () const = 0;
};


class RCC_CoachMetaToken : public RCC_CoachToken
{
public:
  typedef vector < RCC_CoachMetaToken* > container;

  RCC_CoachMetaToken() {}
  virtual ~RCC_CoachMetaToken() {}

  virtual RCC_CoachMetaToken* copy () const = 0;
};

class RCC_CoachMetaToken_Version : public RCC_CoachMetaToken 
{
public:
  RCC_CoachMetaToken_Version ( double v = 0.0 ) : ver_ ( v ) {}
  ~RCC_CoachMetaToken_Version() {}

  ostream& print ( ostream& out ) const { out << "(ver " << ver_ << ")"; return out; }
  ostream& printPretty ( ostream& out, const string& line_header ) const 
    { out << line_header << "version: " << ver_ << endl; return out; }

  RCC_CoachMetaToken* copy () const { return ( RCC_CoachMetaToken* ) new RCC_CoachMetaToken_Version ( *this ); }

  void ver ( const double& v ) { ver_ = v; }
  double ver () const { return ver_; }
      
private:
  double ver_;
};

class RCC_CoachMetaToken_List : public RCC_GCable
{
public:
  RCC_CoachMetaToken_List () {}
  RCC_CoachMetaToken_List ( const RCC_CoachMetaToken_List& l );  
  ~RCC_CoachMetaToken_List () { clear (); }
  
  friend ostream& operator << ( ostream& os, const RCC_CoachMetaToken_List& l)
    { return l.print(os); }
 
  ostream& print ( ostream& out ) const; 
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  unsigned int numOfTokens () { return tokens_.size (); }
  RCC_CoachMetaToken* token ( const unsigned int& i ); 
  void add ( const RCC_CoachMetaToken* t ) { tokens_.push_back ( t->copy () ); }
  void remove ( const unsigned int& i );
  void clear ();

  RCC_CoachMetaToken_List* copy () const { return new RCC_CoachMetaToken_List ( *this ); }
 
protected:
  RCC_CoachMetaToken::container tokens_;

};


class RCC_CoachInfoToken : public RCC_CoachToken
{
public:
  typedef vector < RCC_CoachInfoToken* > container;

  RCC_CoachInfoToken() {}
  virtual ~RCC_CoachInfoToken() {}

  virtual RCC_CoachInfoToken* copy () const = 0;
};


class RCC_CoachInfoToken_Clear : public RCC_CoachInfoToken
{
public:
  RCC_CoachInfoToken_Clear () {}
  ~RCC_CoachInfoToken_Clear () {}

  ostream& print ( ostream& out ) const { out << "(clear)"; return out; }
  ostream& printPretty ( ostream& out, const string& line_header ) const
    { out << line_header << "CLEAR token" << endl; return out; }

  RCC_CoachInfoToken* copy () const { return ( RCC_CoachInfoToken* ) new RCC_CoachInfoToken_Clear ( *this ); }
};


class RCC_CoachInfoToken_Rule : public RCC_CoachInfoToken
{
public:
  RCC_CoachInfoToken_Rule ( const int& ttl, const RCC_Condition* c ); 
  RCC_CoachInfoToken_Rule ( const int& ttl, const RCC_Condition* c, const RCC_DirList& d ); 
  RCC_CoachInfoToken_Rule ( const RCC_CoachInfoToken_Rule& r ); 

  ~RCC_CoachInfoToken_Rule ();

  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  /* class specific stuff */
  int timeToLive () const { return time_to_live; }
  void timeToLive ( const int& ttl ) { time_to_live = ttl; }

  const RCC_Condition* condition () const { return cond_p; }
  void condition ( const RCC_Condition* c ) { delete cond_p; cond_p = c->copy (); }

  RCC_DirList directives() const { return dirs; }
  void directives ( const RCC_DirList* d ); // { dirs = *d; }  

  virtual RCC_CoachInfoToken* copy () const; 

protected:
  int time_to_live;
  RCC_Condition* cond_p;
  RCC_DirList dirs;
};

class RCC_CoachInfoToken_List : public RCC_GCable
{
public:
  RCC_CoachInfoToken_List () {}
  RCC_CoachInfoToken_List ( const RCC_CoachInfoToken_List& l );  
  ~RCC_CoachInfoToken_List () { clear (); }
  
  friend ostream& operator << ( ostream& os, const RCC_CoachInfoToken_List& l)
    { return l.print(os); }
 
  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  unsigned int size () const { return tokens_.size (); }
  RCC_CoachInfoToken* token ( const unsigned int& i ); 
  void add ( const RCC_CoachInfoToken* t ) { tokens_.push_back ( t->copy () ); }
  void remove ( const unsigned int& i );
  void clear ();
 
protected:
  RCC_CoachInfoToken::container tokens_;

};


class RCC_CoachDefineToken : public RCC_CoachToken 
{
public:
  typedef vector < RCC_CoachDefineToken* > container;
  RCC_CoachDefineToken ( const string& n ) : name_ ( n ) {}
  virtual ~RCC_CoachDefineToken() {}

  string name () const { return name_; }
  void  name ( const string& n ) { name_ = n; }

  virtual RCC_CoachDefineToken* copy () const = 0;

protected:
  string name_;
};

class RCC_CoachDefineToken_Condition : public RCC_CoachDefineToken
{
public:
  RCC_CoachDefineToken_Condition ( const string& n, const RCC_Condition* c ) 
    : RCC_CoachDefineToken ( n ), cond_p ( c->copy () ) {}
  RCC_CoachDefineToken_Condition ( const RCC_CoachDefineToken_Condition& t )
    : RCC_CoachDefineToken ( t ), cond_p ( t.cond_p->copy () ) {}
  ~RCC_CoachDefineToken_Condition () { delete cond_p; }

  ostream& print ( ostream& out ) const 
  { out << "(definec \"" << name_ << "\" " << *cond_p << ")"; return out; }
  ostream& printPretty ( ostream& out, const string& line_header ) const
    { out << line_header << "Condition \"" << name_ << "\"" << endl;
    cond_p->printPretty ( out, line_header + " "); return out; }

  RCC_CoachDefineToken* copy () const { return ( RCC_CoachDefineToken* ) new RCC_CoachDefineToken_Condition ( *this ); }

protected:
  RCC_Condition* cond_p;
};

class RCC_CoachDefineToken_Directive : public RCC_CoachDefineToken
{
public:
  RCC_CoachDefineToken_Directive ( const string& n, const RCC_Directive* d )
    : RCC_CoachDefineToken( n ), dir_p ( d->copy () ) {}
  RCC_CoachDefineToken_Directive ( const RCC_CoachDefineToken_Directive& t )
    : RCC_CoachDefineToken( t ), dir_p ( t.dir_p->copy () ) {}
  ~RCC_CoachDefineToken_Directive() { delete dir_p; }

  ostream& print ( ostream& out ) const 
  { out << "(defined \"" << name_ << "\" " << *dir_p << ")"; return out; }
  ostream& printPretty(ostream& out, const string& line_header) const
  { out << line_header << "Directive \"" << name_ << "\"" << endl;
    dir_p->printPretty ( out, line_header + " "); return out; }

  RCC_CoachDefineToken* copy () const { return ( RCC_CoachDefineToken* ) new RCC_CoachDefineToken_Directive ( *this ); }

protected:
  RCC_Directive* dir_p;
};

class RCC_CoachDefineToken_Region : public RCC_CoachDefineToken
{
public:
  RCC_CoachDefineToken_Region ( const string& n, const RCC_Region* r )
    : RCC_CoachDefineToken ( n ), reg_p ( r->copy () ) {}
  RCC_CoachDefineToken_Region ( const RCC_CoachDefineToken_Region& t )
    : RCC_CoachDefineToken ( t ), reg_p ( t.reg_p->copy () ) {}
  ~RCC_CoachDefineToken_Region() { delete reg_p; }

  ostream& print ( ostream& out ) const 
  { out << "(definer \"" << name_ << "\" " << *reg_p << ")"; return out; }
  ostream& printPretty ( ostream& out, const string& line_header ) const
    { out << line_header << "Region \"" << name_ << "\"" << endl;
    reg_p->printPretty ( out, line_header + " " ); return out; }

  RCC_CoachDefineToken* copy () const { return ( RCC_CoachDefineToken* ) new RCC_CoachDefineToken_Region ( *this ); }

protected:
  RCC_Region* reg_p;
};

class RCC_CoachDefineToken_Action : public RCC_CoachDefineToken
{
public:
  RCC_CoachDefineToken_Action ( const string& n, const RCC_Action* a )
    : RCC_CoachDefineToken ( n ), act_p ( a->copy () ) {}
  RCC_CoachDefineToken_Action ( const RCC_CoachDefineToken_Action& t )
    : RCC_CoachDefineToken ( t ), act_p ( t.act_p->copy () ) {}
  ~RCC_CoachDefineToken_Action() { delete act_p; }

  ostream& print ( ostream& out ) const 
  { out << "(definea \"" << name_ << "\" " << *act_p << ")"; return out; }
  ostream& printPretty ( ostream& out, const string& line_header ) const
    { out << line_header << "Action \"" << name_ << "\"" << endl;
    act_p->printPretty ( out, line_header + " "); return out; }

  RCC_CoachDefineToken* copy () const { return ( RCC_CoachDefineToken* ) new RCC_CoachDefineToken_Action ( *this ); }

protected:
  RCC_Action* act_p;
};

class RCC_CoachDefineToken_List : public RCC_GCable
{
public:
  RCC_CoachDefineToken_List () {}
  RCC_CoachDefineToken_List ( const RCC_CoachDefineToken_List& l );  
  ~RCC_CoachDefineToken_List () { clear (); }
  
  friend ostream& operator << ( ostream& os, const RCC_CoachDefineToken_List& l)
  { return l.print(os); }
 
  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  unsigned int size () const { return tokens_.size (); }
  RCC_CoachDefineToken* token ( const unsigned int& i ); 
  void add ( const RCC_CoachDefineToken* t ) { tokens_.push_back ( t->copy () ); }
  void remove ( const unsigned int& i );
  void clear ();
  
protected:
  RCC_CoachDefineToken::container tokens_;

};


#endif
