/* -*- Mode: C++ -*- */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda

    This file is a part of SoccerServer.

    SoccerServer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */


#ifndef _RCC_DIRECTIVE_H_
#define _RCC_DIRECTIVE_H_

#include <stream.h>
#include <vector>


#include "rccaction.h"
#include "rccgarbage.h"
#include "rccside.h"

class RCC_Directive : public RCC_GCable
{
public:
  typedef vector < RCC_Directive* > container;

  RCC_Directive() {}
  virtual ~RCC_Directive() {}
  
  virtual ostream& print ( ostream& out ) const = 0;
  virtual ostream& printPretty ( ostream& out, const string& line_header ) const = 0;

  friend ostream& operator << ( ostream & os, const RCC_Directive& d )
    { return d.print(os); } 

  virtual RCC_Directive* copy () const = 0;
};


class RCC_DirCommand : public RCC_Directive
{
public:
  RCC_DirCommand ( const bool& pos, const RCC_Side& s,
	       const RCC_UnumSet& p, const RCC_Action& a )
    : positive_ ( pos ), side_ ( s ), players ( p ), act_p ( a.copy () ) {}
  RCC_DirCommand ( const RCC_DirCommand& d )
    : positive_ ( d.positive_ ), side_ ( d.side_ ), players ( d.players ), act_p ( d.act_p->copy () ) {}

  ~RCC_DirCommand() { delete act_p; }

  ostream& print ( ostream& out ) const;
  ostream& printPretty ( ostream& out, const string& line_header ) const; 

  bool positive () const { return positive_; }
  bool negative () const { return !positive_; }
  bool ourSide() const { return side_.ourSide (); }
  bool oppSide() const { return side_.oppSide (); }
  const RCC_UnumSet& playerSet() const { return players; }
  RCC_Action* action() const { return act_p; }

  void positive ( const bool& pos ) { positive_ = pos; }
  void side ( const RCC_Side& s ) { side_ = s; }
  void playerSet (const RCC_UnumSet& p ) { players = p; }
  void addPlayer ( const unsigned int& i) { players.add ( i ); }
  void removePlayer ( const unsigned int& i) { players.remove ( i ); }
  void clearPlayers () { players.clear(); }
  void action ( const RCC_Action& act ) { delete act_p; act_p = act.copy (); }
  
  RCC_Directive* copy () const { return ( RCC_Directive* ) new RCC_DirCommand ( *this ); }
protected:
  bool positive_; //true = do; false = dont
  RCC_Side side_; //true = our; false = opp
  RCC_UnumSet players;
  RCC_Action* act_p;
} ;

class RCC_DirNamed : public RCC_Directive
{
public:
  RCC_DirNamed ( const string& n ) : name_ ( n ) {}
  ~RCC_DirNamed() {}

  ostream& print ( ostream& out ) const { out << "\"" << name_ << "\""; return out; }
  ostream& printPretty ( ostream& out, const string& line_header ) const
    { out << line_header << "directived named \"" << name_ << "\"" << endl; return out; }

  /* class specific */
  string name () const { return name_; }
  void  name ( const string& n ) { name_ = n; } 

  RCC_Directive* copy () const { return ( RCC_Directive* ) new RCC_DirNamed ( *this ); }
protected:
  string name_;
};

class RCC_DirList : public RCC_GCable
{
public:
  RCC_DirList () {}
  RCC_DirList ( const RCC_DirList& d );
  ~RCC_DirList () { clear (); }

  friend ostream& operator << ( ostream & os, const RCC_DirList& d )
    { return d.print(os); } 

  ostream& print ( ostream& out ) const; 
  ostream& printPretty ( ostream& out, const string& line_header ) const;

  unsigned int numOfDirectives () { return dirs.size (); }
  RCC_Directive* directive ( const unsigned int& i ); 
  void add ( const RCC_Directive* d ) { dirs.push_back ( d->copy () ); }
  void remove ( const unsigned int& i );
  void clear ();

  RCC_DirList& operator= ( const RCC_DirList& l );

protected:
  RCC_Directive::container dirs;
};


#endif
