/* -*-c++-*- */

#ifndef _RCC_GARBAGE_H_
#define _RCC_GARBAGE_H_

#include <list>

class RCC_GCable
{
public:
  RCC_GCable ();
  virtual ~RCC_GCable ();
};


class RCC_GCElement
{
public:
  RCC_GCElement () {}
  virtual ~RCC_GCElement ();
  virtual void delElem () = 0;

  friend bool operator== ( const RCC_GCElement& a, const RCC_GCElement& b) { return a.itemPtr () == b.itemPtr (); }
  virtual const void* itemPtr () const = 0;

};

//===============================================
//
//  RCC_GCElement_GCable
//
//===============================================

class RCC_GCElement_GCable : public RCC_GCElement
{
public:
  RCC_GCElement_GCable ( RCC_GCable* item_p ) : ptr ( item_p ) {}
  virtual ~RCC_GCElement_GCable ();
  virtual void delElem ();
  virtual const void* itemPtr () const;
  
protected:
  RCC_GCable* ptr;
};

//===============================================
//
//  RCC_GCElement_DoubleArr
//
//===============================================

class RCC_GCElement_DoubleArr : public RCC_GCElement
{
public:
  RCC_GCElement_DoubleArr ( double* arr_p ) : ptr ( arr_p ) {}
  virtual ~RCC_GCElement_DoubleArr ();
  virtual void delElem ();
  virtual const void* itemPtr () const;

protected:
  double* ptr;
};


//===============================================
//
//  RCC_GarbageCollector
//
//===============================================

class RCC_GarbageCollector
{
public:
  RCC_GarbageCollector () {}
  ~RCC_GarbageCollector () { delAll (); }

  void delAll ();

  void add ( RCC_GCElement* elem ) { elem_list.push_back ( elem ); }
  void add ( RCC_GCable* elem ) { add ( new RCC_GCElement_GCable ( elem ) ); }
  void add ( double* elem ) { add ( new RCC_GCElement_DoubleArr ( elem ) ); }

  bool remove ( const RCC_GCElement& elem );
  bool remove ( RCC_GCable* elem ) { return remove ( RCC_GCElement_GCable ( elem ) ); }
  bool remove ( double* elem ) { return remove ( RCC_GCElement_DoubleArr ( elem ) ); } 

  bool remDel ( const RCC_GCElement& elem );
  bool remDel ( RCC_GCable* elem ) { return remDel ( RCC_GCElement_GCable ( elem ) ); }
  bool remDel ( double* elem ) { return remDel ( RCC_GCElement_DoubleArr ( elem ) ); } 

protected:
  list < RCC_GCElement* > elem_list;
};


#endif
