// -*-c++-*-
#ifndef _RCC_OBJECT_POOL_H_
#define _RCC_OBJECT_POOL_H_

#include "rccobject.h"
#include <vector>
#include "rccgarbage.h"
#include "rccsditem.h"

class RCC_ObjectPool : public RCC_GCable, public RCC_SD_Item
{
public:
  RCC_ObjectPool ( const string& tn );
  ~RCC_ObjectPool ();
  
  void clear ();
  
  int time () const { return op_time; }
  int time ( const int &time ) { return (op_time = time); }

  void add ( const RCC_Object* obj );
  void add ( const RCC_Line& line );
  void add ( const RCC_Flag& flag );
  void add ( const RCC_Player& player );
  void add ( const RCC_Ball& ball );
    
  friend ostream& operator<<(ostream& o, const RCC_ObjectPool& op) { return op.print(o); }
  ostream& print ( ostream& o ) const
  {
    return print ( o, RCC_Vector2d::POLAR, RCC_Angle::DEG );
  }
  ostream& print ( ostream& o, const Side& ) const { return print ( o ); } 

  ostream& print ( ostream& o, RCC_Vector2d::mode_t vm, RCC_Angle::mode_t am ) const;

  RCC_SD_Item* copy () const { return ( RCC_SD_Item* ) new RCC_ObjectPool ( *this ); }

  int numOfBalls () const { return balls.size (); } 
  const RCC_Ball& ball ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfBalls () ) 
      return balls [ idx ]; 
    throw ( "Ball idx out of bounds" );
  }

  int numOfLines () const { return lines.size (); } 
  const RCC_Line& line ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfLines () ) 
      return lines [ idx ]; 
    throw ( "Line idx out of bounds" );
  }

  int numOfFlags () const { return flags.size (); } 
  const RCC_Flag& flag ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfFlags () ) 
      return flags [ idx ]; 
    throw ( "Flag idx out of bounds" ); 
  }

  int numOfGoals () const { return goals.size (); } 
  const RCC_Flag& goal ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfGoals () ) 
      return goals [ idx ]; 
    throw ( "Goal idx out of bounds" ); 
  }


  int numOfPlayers () const { return numOfAllies () + numOfNeuts () + numOfEnemies (); } 
  const RCC_Player& player ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfAllies () ) 
      return ally ( idx );

    else if ( idx - numOfAllies () < numOfNeuts () )
      return neut ( idx - numOfAllies () );

    else if ( idx - numOfAllies () - numOfNeuts () < numOfEnemies () )
      return enemy ( idx - numOfAllies () - numOfNeuts () );

    throw ( "Player idx out of bounds" ); 
  }

  int numOfAllies () const { return allies.size (); } 
  const RCC_Player& ally ( unsigned int idx = 0 ) const 
  { 
    if ( idx < allies.size () ) 
      return allies [ idx ]; 
    throw ( "Ally idx out of bounds" ); 
  }

  int numOfNeuts () const { return neuts.size (); } 
  const RCC_Player& neut ( unsigned int idx = 0 ) const 
  { 
    if ( idx < neuts.size () ) 
      return neuts [ idx ]; 
    throw ( "Neut idx out of bounds" ); 
  }

  int numOfEnemies () const { return enemies.size (); } 
  const RCC_Player& enemy ( unsigned int idx = 0 ) const 
  { 
    if ( idx < enemies.size () ) 
      return enemies [ idx ]; 
    throw ( "Enemy idx out of bounds" ); 
  }

    
private:
  string team_name;

  int op_time;
    
  vector < RCC_Line > lines;	// 0, 1 or many
  vector < RCC_Flag > flags;	// 0, 1 or many
  vector < RCC_Flag > goals;	// 0, 1 or 2
  vector < RCC_Player > allies;	// 0, 1 or many
  vector < RCC_Player > enemies;// 0, 1 or many
  vector < RCC_Player > neuts;	// 0, 1 or many
  vector < RCC_Ball > balls;	// 0 or 1

};

#endif
