// -*-c++-*-
#ifndef _RCC_PLAY_MODE_H_
#define _RCC_PLAY_MODE_H_

#include "param.h"
#include "types.h"
#include "rccside.h"
#include "rccgarbage.h"

#include <string>
#include <iostream.h>


class RCC_PlayMode : public RCC_GCable
{
public:
  enum mode_t {
    Unknown,
    BeforeKickOff,
    TimeOver,
    PlayOn,
    KickOff,
    KickIn,
    FreeKick,
    CornerKick,
    GoalKick,
    AfterGoal,
    DropBall,
    OffSide,
    PK,
    FirstHalfOver,
    Pause,
    Human,
    FoulCharge,
    FoulPush,
    FoulMultipleAttacker,
    FoulBallOut,
    GoalieCatch,
    TimeUpWithoutTeam,
    TimeUp,
    TimeExtended,
    HalfTime,
    Foul,
    Goal,
  };
  
  typedef PlayMode stad_mode_t;

  enum coach_mode_t {
    none,
    bko,
    time_over,
    play_on,
    ko_our,		
    ko_opp,		
    ki_our,		
    ki_opp,	
    fk_our,
    fk_opp,
    ck_our,
    ck_opp,
    gk_our,
    gk_opp,
    gc_our,
    gc_opp,
    ag_our,
    ag_opp
  };
  
  static const string STAD_NAMES[];
  static const string COACH_NAMES[];
  static const string NAMES[];
  
  RCC_PlayMode ( const RCC_PlayMode::mode_t& m, const RCC_Side& s )
    : mode_ ( m ), side_ ( s ) {}

  RCC_PlayMode ( const RCC_PlayMode::stad_mode_t& m, const Side& team_side )
  { mode ( m, team_side ); }
  
  RCC_PlayMode ( const RCC_PlayMode::coach_mode_t& m )
  { mode ( m ); }
  
  ~RCC_PlayMode () {}

  RCC_Side side () const { return side_; }
  RCC_Side side ( const RCC_Side& s ) { return side_ = s; }
    
  RCC_PlayMode::mode_t mode () const { return mode_; }
  RCC_PlayMode::stad_mode_t stadMode ( const Side& s ) const;
  RCC_PlayMode::coach_mode_t coachMode () const;

  RCC_PlayMode::mode_t mode ( const RCC_PlayMode::mode_t& m ) { return mode_ = m; }

  // the two methods below will set the playmode based on a server playmode.  The side
  // remains unchanged unless the team_side parameter is provided
  RCC_PlayMode::mode_t mode ( const RCC_PlayMode::stad_mode_t& m, const Side& team_side );  
  RCC_PlayMode::mode_t mode ( const RCC_PlayMode::stad_mode_t& m );

  // sets the playmode and side based on a coach msg playmode
  RCC_PlayMode::mode_t mode ( const RCC_PlayMode::coach_mode_t& m );

  int points () const { return points_; }
  int points ( const int& p ) { return points_ = p; }
 

  friend ostream& operator<< ( ostream& o, const RCC_PlayMode& pm )
  {
    return pm.print ( o );
  }

  ostream& print ( ostream& o ) const
  { return o << RCC_PlayMode::NAMES [ mode_ ] << " " << side_; }

protected:
  RCC_PlayMode::mode_t	mode_;
  RCC_Side		side_;
  int			points_;
};

ostream& operator<< ( ostream& o, const PlayMode& pm );

ostream& operator<< ( ostream& o, const RCC_PlayMode::coach_mode_t& pm );

#endif
