// -*-c++-*-

#ifndef _RCC_SERVER_PARAM_H_
#define _RCC_SERVER_PARAM_H_

#include "rccangle.h"
#include "rccgarbage.h"
#include "rccsditem.h"

class RCC_ServerParam : public RCC_GCable, public RCC_SD_Item
{
public:
    RCC_ServerParam ( const double& par_gwidth, 
		  const double& par_inertia_moment, 
		  const double& par_psize, 
		  const double& par_pdecay, 
		  const double& par_prand,
		  const double& par_pweight, 
		  const double& par_pspeed_max, 
		  const double& par_paccel_max, 
		  const double& par_stamina_max, 
		  const double& par_stamina_inc,
		  const double& par_recover_init, 
		  const double& par_recover_dthr, 
		  const double& par_recover_min, 
		  const double& par_recover_dec, 
		  const double& par_effort_init,
		  const double& par_effort_dthr, 
		  const double& par_effort_min, 
		  const double& par_effort_dec, 
		  const double& par_effort_ithr, 
		  const double& par_effort_inc,
		  const double& par_kick_rand, 
		  const int& par_team_actuator_noise, 
		  const double& par_prand_factor_l, 
		  const double& par_prand_factor_r, 
		  const double& par_kick_rand_factor_l,
		  const double& par_kick_rand_factor_r, 
		  const double& par_bsize, 
		  const double& par_bdecay, 
		  const double& par_brand, 
		  const double& par_bweight,
		  const double& par_bspeed_max, 
		  const double& par_baccel_max, 
		  const double& par_dprate, 
		  const double& par_kprate, 
		  const double& par_kmargin,
		  const double& par_ctlradius, 
		  const double& par_ctlradius_width, 
		  const double& par_maxp, 
		  const double& par_minp, 
		  const double& par_maxm,
		  const double& par_minm, 
		  const double& par_maxnm, 
		  const double& par_minnm, 
		  const double& par_maxn, 
		  const double& par_minn,
		  RCC_Angle par_visangle, 
		  const double& par_visdist, 
		  RCC_Angle par_windir, 
		  const double& par_winforce, 
		  const double& par_winang,
		  const double& par_winrand, 
		  const double& par_kickable_area, 
		  const double& par_catch_area_l, 
		  const double& par_catch_area_w, 
		  const double& par_catch_prob,
		  const int& par_goalie_max_moves, 
		  const double& par_ckmargin, 
		  const double& par_offside_area, 
		  const int& par_win_no, 
		  const int& par_win_random,
		  const int& par_say_cnt_max, 
		  const int& par_SayCoachMsgSize, 
		  const int& par_clang_win_size, 
		  const int& par_clang_define_win, 
		  const int& par_clang_meta_win,
		  const int& par_clang_advice_win, 
		  const int& par_clang_info_win, 
		  const int& par_clang_mess_delay, 
		  const int& par_clang_mess_per_cycle, 
		  const int& par_half_time,
		  const int& par_sim_st, 
		  const int& par_send_st, 
		  const int& par_recv_st, 
		  const int& par_sb_step, 
		  const int& par_lcm_st,
		  const int& par_SayMsgSize, 
		  const int& par_hear_max, 
		  const int& par_hear_inc, 
		  const int& par_hear_decay, 
		  const int& par_cban_cycle,
		  const int& par_slow_down_factor, 
		  const int& par_useoffside, 
		  const int& par_kickoffoffside, 
		  const double& par_offside_kick_margin, 
		  const double& par_audio_dist,
		  const double& par_dist_qstep, 
		  const double& par_land_qstep, 
		  const double& par_dir_qstep, 
		  const double& par_dist_qstep_l, 
		  const double& par_dist_qstep_r,
		  const double& par_land_qstep_l, 
		  const double& par_land_qstep_r, 
		  const double& par_dir_qstep_l, 
		  const double& par_dir_qstep_r, 
		  const int& par_CoachMode,
		  const int& par_CwRMode, 
		  const int& par_old_hear, 
		  const int& par_sv_st, 
		  const int& par_start_goal_l, 
		  const int& par_start_goal_r,
		  const int& par_fullstate_l, 
		  const int& par_fullstate_r, 
		  const int& par_drop_time);

  ~RCC_ServerParam () {}
   
  friend ostream& operator<< (ostream& o, const RCC_ServerParam& sp)
  {
    return sp.print ( o );
  }

  RCC_SD_Item* copy () const { return ( RCC_SD_Item* ) new RCC_ServerParam ( *this ); }

  ostream& print ( ostream& o ) const;
  ostream& print ( ostream& o, const Side& ) const { return print ( o ); } 

private:
    double gwidth; 					
    double inertia_moment; 	
    double psize; 			
    double pdecay; 			
    double prand; 
			
    double pweight; 				
    double pspeed_max; 		
    double paccel_max; 		
    double stamina_max; 			
    double stamina_inc; 
			
    double recover_init; 		
    double recover_dthr; 		
    double recover_min; 			
    double recover_dec; 			
    double effort_init; 
			
    double effort_dthr; 			
    double effort_min; 		
    double effort_dec; 			
    double effort_ithr; 			
    double effort_inc;
 			
    double kick_rand;                             
    int team_actuator_noise;                     
    double prand_factor_l;                          
    double prand_factor_r;                            
    double kick_rand_factor_l;
                       
    double kick_rand_factor_r;                     
    double bsize; 			
    double bdecay; 			
    double brand; 			
    double bweight;
 				
    double bspeed_max; 			
    double baccel_max; 			
    double dprate; 			
    double kprate; 			
    double kmargin;
 				
    double ctlradius; 			
    double ctlradius_width; 		
    double maxp;				
    double minp; 				
    double maxm;
 				
    double minm; 				
    double maxnm; 				
    double minnm; 				
    double maxn; 				
    double minn;
 				
    RCC_Angle visangle; 			
    double visdist; 					
    RCC_Angle windir; 					
    double winforce; 				
    double winang;
 					
    double winrand; 					
    double kickable_area; 			
    double catch_area_l; 			
    double catch_area_w; 			
    double catch_prob; 
				
    int goalie_max_moves;                  	
    double ckmargin; 				
    double offside_area; 			
    int win_no; 					
    int win_random; 
				
    int say_cnt_max; 				
    int SayCoachMsgSize; 				
    int clang_win_size;
    int clang_define_win;
    int clang_meta_win;

    int clang_advice_win;
    int clang_info_win;
    int clang_mess_delay;
    int clang_mess_per_cycle;
    int half_time;
 					
    int sim_st; 					
    int send_st; 					
    int recv_st; 					
    int sb_step; 					
    int lcm_st;
 		                        
    int SayMsgSize; 				
    int hear_max; 					
    int hear_inc; 					
    int hear_decay; 				
    int cban_cycle;
 				
    int slow_down_factor;                           
    int useoffside; 				
    int kickoffoffside; 			
    double offside_kick_margin; 		
    double audio_dist; 
				
    double dist_qstep; 				
    double land_qstep; 				
    double dir_qstep; 				
    double dist_qstep_l; 			
    double dist_qstep_r; 
			
    double land_qstep_l; 			
    double land_qstep_r; 			
    double dir_qstep_l; 				
    double dir_qstep_r; 				
    int CoachMode;
 				
    int CwRMode; 					
    int old_hear; 					
    int sv_st;
    int start_goal_l;
    int start_goal_r;

    int fullstate_l;
    int fullstate_r;
    int drop_time;
};


#endif
