// -*-c++-*-
#ifndef _RCC_SIDE_H_
#define _RCC_SIDE_H_

#include "param.h"
#include "types.h"
#include "rccgarbage.h"

#include <string>
#include <iostream.h>

class RCC_Side : public RCC_GCable
{
public:
  enum mode_t { Unknown, Our, Opp, Neut };
  static const string NAMES [];

  RCC_Side () : mode_ ( RCC_Side::Unknown ) {}
  RCC_Side ( const RCC_Side::mode_t& m ) : mode_ ( m ) {}
  RCC_Side ( const Side& s, const Side& team_side ) { mode ( s, team_side ); }
  ~RCC_Side () {}

  RCC_Side::mode_t mode () const { return mode_; }
  RCC_Side::mode_t mode ( const RCC_Side::mode_t m ) { return mode_ = m; }
  
  Side stadMode ( const Side& team_side ) const
  {
    switch ( mode_ )
      {
      case RCC_Side::Unknown:
      case RCC_Side::Neut:
	return NEUTRAL;

      case RCC_Side::Our:
	return team_side;

      case RCC_Side::Opp:
	return team_side * -1;
      }
  }

  RCC_Side::mode_t mode ( const Side& s, const Side& team_side ) 
  {
    switch ( s )
      {
      case NEUTRAL:
	return mode_ = RCC_Side::Neut;
      case LEFT:
	switch ( team_side )
	  {
	  case LEFT:
	    return mode_ = RCC_Side::Our;
	  case RIGHT:
	    return mode_ = RCC_Side::Opp;
	  default:
	    return mode_ = RCC_Side::Unknown;
	  }
      case RIGHT:
	switch ( team_side )
	  {
	  case LEFT:
	    return mode_ = RCC_Side::Opp;
	  case RIGHT:
	    return mode_ = RCC_Side::Our;
	  default:
	    return mode_ = RCC_Side::Unknown;
	  }
      default:
	return mode_ = RCC_Side::Unknown;
      }
  }

  friend ostream& operator<< ( ostream& o, const RCC_Side& s )
  {
    return s.print ( o );
  }

  ostream& print ( ostream& o ) const
  {
    return o << RCC_Side::NAMES [ mode_ ];
  }

  bool ourSide () const { return mode_ == RCC_Side::Our; }
  bool oppSide () const { return mode_ == RCC_Side::Opp; }
  bool neutSide () const { return mode_ == RCC_Side::Neut; }

protected:
  RCC_Side::mode_t mode_;
};



#endif
