#include "rccangle.h"

RCC_Angle::RCC_Angle () : valid ( false )
{
}

RCC_Angle::RCC_Angle ( const double& )
{
  throw "Must Specify mode when constructing an RCC_Angle";
}

RCC_Angle::RCC_Angle ( const double& d, const RCC_Angle::mode_t& m ) 
  : valid ( true ),  mode ( m )
{
  switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value = d;
	  break;

	case RCC_Angle::RAD:
	  rad_value = d;
	  break;

	case RCC_Angle::BOTH:
	  valid = false;
	  throw ("Can not initialise RCC_Angle with a mode of RCC_Angle::BOTH"); 
	  break;
	}
}

const RCC_Angle& RCC_Angle::norm()	
{
  if ( valid )
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value = normDeg ( deg_value );
	  break;

	case RCC_Angle::RAD:
	  rad_value = normRad ( rad_value );
	  break;

	case RCC_Angle::BOTH:
	  deg_value = normDeg ( deg_value );
	  mode = RCC_Angle::DEG;
	  break;
	}
    }
  return *this;
}

const RCC_Angle& RCC_Angle::trunc()	
{
  if ( valid )
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value = RCC_Angle::truncDeg ( deg_value );
	  break;

	case RCC_Angle::RAD:
	  rad_value = RCC_Angle::truncRad ( rad_value );
	  break;

	case RCC_Angle::BOTH:
	  deg_value = RCC_Angle::truncDeg ( deg_value );
	  mode = RCC_Angle::DEG;
	  break;
	}
    }
  return *this;
}

ostream& RCC_Angle::print ( ostream& o ) const
{ 
  return print ( o, RCC_Angle::DEG ); 
}

ostream& RCC_Angle::print ( ostream& o, RCC_Angle::mode_t m ) const
{ 
  if ( valid )
    {
      if ( m == RCC_Angle::DEG )
	return o << deg ();
      else
	return o << rad ();
    }
  else
    return o << "unknown"; 
}

ostream& RCC_Angle::printPretty ( ostream& o, RCC_Angle::mode_t m ) const
{ 
  if ( valid )
    {
      if ( m == RCC_Angle::DEG )
	{
	  double val1 = deg();

	  if ( val1 < 0.0 )
	    {
	      val1 = - val1;
	      o << "-";
	    }
    
	  double val2 = ( val1 - floor ( val1 ) ) * 60.0;
	  int val3 = ( int ) rint( ( val2 - floor ( val2 ) ) * 60.0 );

	  o << floor ( val1 ) << DEGREE_SYMBOL;

	  if ( val2 != 0.0 || val3 != 0 )
	    o << floor ( val2 ) << "'";

	  if ( val3 != 0 )
	    o << val3 << "\"";
	}
      else
	o << rad () << "RAD";
    }
  else
    o << "unknown" << DEGREE_SYMBOL;

  return o; 
}


bool equals(const RCC_Angle& a, const RCC_Angle& b)
{
  switch ( a.mode )
    {
    case RCC_Angle::DEG:
      return ( a.deg_value == b.deg () );
      
    case RCC_Angle::RAD:
      return ( a.rad_value == b.rad () );

    case RCC_Angle::BOTH:
      switch ( b.mode )
	{
	case RCC_Angle::DEG:
	  return ( a.deg_value == b.deg_value );

	case RCC_Angle::RAD:
	case RCC_Angle::BOTH:
	  return ( a.rad_value == b.rad_value );
	}
    }
}


RCC_Angle RCC_Angle::asin(const double &i)
{
    if (i < -1.0 || i > 1.0)
	return RCC_Angle();
    else
	return RCC_Angle ( ::asin(i), RCC_Angle::RAD);
}

RCC_Angle RCC_Angle::acos(const double &i)
{
    if (i < -1.0 || i > 1.0)
	return RCC_Angle();
    else
	return RCC_Angle( ::acos(i), RCC_Angle::RAD);
}

RCC_Angle RCC_Angle::atan(const double &i)
{
    return RCC_Angle( ::atan(i), RCC_Angle::RAD);
}

RCC_Angle RCC_Angle::atan2(const double &y, const double&x)
{
    return RCC_Angle( ::atan2( y, x ), RCC_Angle::RAD);
}

RCC_Angle& RCC_Angle::operator+= (const RCC_Angle& ang)
{
  if ( valid && ang.valid )
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value += ang.deg ();
	  break;

	case RCC_Angle::RAD:
	  rad_value += ang.rad ();
	  break;

	case RCC_Angle::BOTH:
	  switch ( ang.mode )
	    {
	    case RCC_Angle::DEG:
	      deg_value += ang.deg_value;
	      mode = RCC_Angle::DEG;
	      break;

	    case RCC_Angle::RAD:
	      rad_value += ang.rad_value;
	      mode = RCC_Angle::RAD;
	      break;

	    case RCC_Angle::BOTH:
	      deg_value += ang.deg_value;
	      rad_value += ang.rad_value;
	      break;
	    }
	  break;
	}
    }
  else // not valid
    valid = false;

  return *this;
}

RCC_Angle& RCC_Angle::operator-= (const RCC_Angle& ang)
{
  if ( valid && ang.valid )
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value -= ang.deg ();
	  break;

	case RCC_Angle::RAD:
	  rad_value -= ang.rad ();
	  break;

	case RCC_Angle::BOTH:
	  switch ( ang.mode )
	    {
	    case RCC_Angle::DEG:
	      deg_value -= ang.deg_value;
	      mode = RCC_Angle::DEG;
	      break;

	    case RCC_Angle::RAD:
	      rad_value -= ang.rad_value;
	      mode = RCC_Angle::RAD;
	      break;

	    case RCC_Angle::BOTH:
	      deg_value -= ang.deg_value;
	      rad_value -= ang.rad_value;
	      break;
	    }
	  break;
	}
    }
  else // not valid
    valid = false;

  return *this;
}


RCC_Angle& RCC_Angle::operator*= (const double& s)
{
  if ( s == HUGE_VAL )
    valid = false;
  else
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value *= s;
	  break;

	case RCC_Angle::RAD:
	  rad_value *= s;
	  break;

	case RCC_Angle::BOTH:
	  deg_value *= s;
	  rad_value *= s;
	  break;
	}
    }
  return *this;
}

RCC_Angle& RCC_Angle::operator/= (const double& s)
{
  if ( s == 0.0 )
    valid = false;
  else
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  deg_value /= s;
	  break;

	case RCC_Angle::RAD:
	  rad_value /= s;
	  break;

	case RCC_Angle::BOTH:
	  deg_value /= s;
	  rad_value /= s;
	  break;
	}
    }
  return *this;
}

RCC_Angle RCC_Angle::operator-() const 
{
  if ( valid )
    {
      switch ( mode )
	{
	case RCC_Angle::DEG:
	  return RCC_Angle (-deg_value, RCC_Angle::DEG);

	case RCC_Angle::RAD:
	case RCC_Angle::BOTH:
	  return RCC_Angle (-rad_value, RCC_Angle::RAD);
	}
    }
  return *this;
}
