#include "rcccoachtoken.h"

RCC_CoachMetaToken_List::RCC_CoachMetaToken_List ( const RCC_CoachMetaToken_List& d )
{
  for ( RCC_CoachMetaToken::container::const_iterator i = d.tokens_.begin(); i != d.tokens_.end(); i++ )
    tokens_.push_back ( ( *i )->copy () );
}

ostream& RCC_CoachMetaToken_List::print ( ostream& out ) const
{
  string space = "";
  for ( RCC_CoachMetaToken::container::const_iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    {
      out << space << **i;
      space = " ";
    }
  return out;
}
 
ostream& RCC_CoachMetaToken_List::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header << "MetaToken list\n";
  for ( RCC_CoachMetaToken::container::const_iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    ( *i )->printPretty ( out, line_header + "- " );

  return out;
}
 


RCC_CoachMetaToken* RCC_CoachMetaToken_List::token ( const unsigned int& i )
{
  if ( i < tokens_.size () )
    return tokens_ [ i ];
  else
    throw ( "RCC_CoachMetaTokenList idx out of bounds" );
}

void RCC_CoachMetaToken_List::remove ( const unsigned int& i )
{
  unsigned int tmp = 0; 
  for ( RCC_CoachMetaToken::container::iterator j = tokens_.begin(); j != tokens_.end(); j++ )
    {
      if ( i == tmp )
	{
	tokens_.erase ( j );
	delete *j;
	return;
	}
      tmp++;
    }
  throw ( "RCC_CoachMetaTokenList idx out of bounds" );
}

void RCC_CoachMetaToken_List::clear ()
{
  for ( RCC_CoachMetaToken::container::iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    delete *i;

  tokens_.clear ();
} 

RCC_CoachInfoToken_Rule::RCC_CoachInfoToken_Rule ( const int& ttl, const RCC_Condition* c ) 
  : time_to_live ( ttl ), cond_p ( c->copy () ) 
{
}

RCC_CoachInfoToken_Rule::RCC_CoachInfoToken_Rule ( const int& ttl, const RCC_Condition* c, const RCC_DirList& d ) 
  : time_to_live ( ttl ), cond_p ( c->copy () ), dirs ( d ) 
{
}

RCC_CoachInfoToken_Rule::RCC_CoachInfoToken_Rule ( const RCC_CoachInfoToken_Rule& r ) 
  : time_to_live ( r.time_to_live ), cond_p ( r.cond_p->copy () ), dirs ( r.dirs ) 
{
}

RCC_CoachInfoToken_Rule::~RCC_CoachInfoToken_Rule () 
{ 
delete cond_p; 
}

ostream& RCC_CoachInfoToken_Rule::print ( ostream& out ) const
{
  out << "(" << time_to_live << " " << *cond_p << " " << dirs << ")";
  return out;
}

ostream& RCC_CoachInfoToken_Rule::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header << "for " << time_to_live << " cycles, if" << endl;
  cond_p->printPretty ( out, line_header + " +" ) << line_header << "then" << endl;
  dirs.printPretty ( out, line_header + " *" );
  
  return out;
}


void RCC_CoachInfoToken_Rule::directives ( const RCC_DirList* d ) 
{ 
    dirs = *d; 
}  

RCC_CoachInfoToken* RCC_CoachInfoToken_Rule::copy () const 
{ 
  return ( RCC_CoachInfoToken* ) new RCC_CoachInfoToken_Rule ( *this );
}


RCC_CoachInfoToken_List::RCC_CoachInfoToken_List ( const RCC_CoachInfoToken_List& l )
{
  for ( RCC_CoachInfoToken::container::const_iterator i = l.tokens_.begin(); i != l.tokens_.end(); i++ )
    tokens_.push_back ( ( *i )->copy () );
}

ostream& RCC_CoachInfoToken_List::print ( ostream& out ) const
{
  string space = "";
  for ( RCC_CoachInfoToken::container::const_iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    {
      out << space << **i;
      space = " ";
    }
  return out;
}
 
ostream& RCC_CoachInfoToken_List::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header << "InfoToken list\n";
  for ( RCC_CoachInfoToken::container::const_iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    ( *i )->printPretty ( out, line_header + "- " );

  return out;
}
 

RCC_CoachInfoToken* RCC_CoachInfoToken_List::token ( const unsigned int& i )
{
  if ( i < tokens_.size () )
    return tokens_ [ i ];
  else
    throw ( "RCC_CoachInfoToken_List idx out of bounds" );
}

void RCC_CoachInfoToken_List::remove ( const unsigned int& i )
{
  unsigned int tmp = 0; 
  for ( RCC_CoachInfoToken::container::iterator j = tokens_.begin(); j != tokens_.end(); j++ )
    {
      if ( i == tmp )
	{
	tokens_.erase ( j );
	delete *j;
	return;
	}
      tmp++;
    }
  throw ( "RCC_CoachInfoTokenList idx out of bounds" );
}

void RCC_CoachInfoToken_List::clear ()
{
  for ( RCC_CoachInfoToken::container::iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    delete *i;

  tokens_.clear ();
} 


RCC_CoachDefineToken_List::RCC_CoachDefineToken_List ( const RCC_CoachDefineToken_List& l )
{
  for ( RCC_CoachDefineToken::container::const_iterator i = l.tokens_.begin(); i != l.tokens_.end(); i++ )
    tokens_.push_back ( ( *i )->copy () );
}

ostream& RCC_CoachDefineToken_List::print ( ostream& out ) const
{
  string space ( "" );
  for ( RCC_CoachDefineToken::container::const_iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    {
      out << space << **i;
      space = " ";
    }
  return out;
}
 
ostream& RCC_CoachDefineToken_List::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header << "DefineToken list\n";
  for ( RCC_CoachDefineToken::container::const_iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    ( *i )->printPretty ( out, line_header + "- " );

  return out;
}
 

RCC_CoachDefineToken* RCC_CoachDefineToken_List::token ( const unsigned int& i )
{
  if ( i < tokens_.size () )
    return tokens_ [ i ];
  else
    throw ( "RCC_CoachDefineToken_List idx out of bounds" );
}

void RCC_CoachDefineToken_List::remove ( const unsigned int& i )
{
  unsigned int tmp = 0; 
  for ( RCC_CoachDefineToken::container::iterator j = tokens_.begin(); j != tokens_.end(); j++ )
    {
      if ( i == tmp )
	{
	tokens_.erase ( j );
	delete *j;
	return;
	}
      tmp++;
    }
  throw ( "RCC_CoachDefineTokenList idx out of bounds" );
}

void RCC_CoachDefineToken_List::clear ()
{
  for ( RCC_CoachDefineToken::container::iterator i = tokens_.begin(); i != tokens_.end(); i++ )
    delete *i;

  tokens_.clear ();
} 


