#include "rccdirective.h"

ostream& RCC_DirCommand::print ( ostream& out ) const
{
  out << "("
      << (positive_ ? "do" : "dont") << " "
      << side_ << " "
      << players << " "
      << *act_p << ")";
  return out; 
}

ostream& RCC_DirCommand::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header 
      << side_ << " "
      << players << " "
      << (positive_ ? "DO" : "DON'T") << ": " << endl;
  return act_p->printPretty ( out, line_header + " ");
}

RCC_DirList::RCC_DirList ( const RCC_DirList& d )
{
  for ( RCC_Directive::container::const_iterator i = d.dirs.begin(); i != d.dirs.end(); i++ )
    dirs.push_back ( ( *i )->copy () );
}

RCC_Directive* RCC_DirList::directive ( const unsigned int& i )
{
  if ( i < dirs.size () )
    return dirs [ i ];
  else
    throw ( "RCC_DirList idx out of bounds" );
}

void RCC_DirList::remove ( const unsigned int& i )
{
  unsigned int tmp = 0; 
  for ( RCC_Directive::container::iterator j = dirs.begin(); j != dirs.end(); j++ )
    {
      if ( i == tmp )
	{
	dirs.erase ( j );
	delete *j;
	return;
	}
      tmp++;
    }
  throw ( "RCC_DirList idx out of bounds" );
}

void RCC_DirList::clear ()
{
  for ( RCC_Directive::container::iterator i = dirs.begin(); i != dirs.end(); i++ )
    delete *i;

  dirs.clear ();
}

ostream& RCC_DirList::print ( ostream& out ) const
{
  string space = "";
  for ( RCC_Directive::container::const_iterator i = dirs.begin(); i != dirs.end(); i++ )
    {
      out << space << **i;
      space = " ";
    }

  return out;
} 

ostream& RCC_DirList::printPretty ( ostream& out, const string& line_header ) const
{
  for ( RCC_Directive::container::const_iterator i = dirs.begin(); i != dirs.end(); i++ )
    ( *i )->printPretty ( out, line_header );

  return out;
}

RCC_DirList& RCC_DirList::operator= ( const RCC_DirList& l )
{
  clear ();
  for ( RCC_Directive::container::const_iterator i = l.dirs.begin(); i != l.dirs.end(); i++ )
    dirs.push_back ( ( *i )->copy () );  
  return *this;
}
