#include "rccobject.h"


//=======================================================================
//	RCC_Object
//=======================================================================

const string RCC_Object::LOCATION_NAMES[] = { "INVALID", "t", "b", "l", "r", "c", "p", "g", "INVALID" };

RCC_Object::RCC_Object ( const RCC_Vector2d& pos ) : rel_pos ( pos )
{
}

RCC_Object::RCC_Object ( const RCC_Angle& dir ) : rel_pos ( HUGE_VAL, dir )
{
}

RCC_Object::~RCC_Object()
{
}


//=======================================================================
//	RCC_StatObject
//=======================================================================

RCC_StatObject::RCC_StatObject ( const RCC_Vector2d& pos ) : RCC_Object ( pos )
{
}

RCC_StatObject::RCC_StatObject ( const RCC_Angle& dir ) : RCC_Object ( dir )
{
}

RCC_StatObject::~RCC_StatObject()
{
}


//=======================================================================
//	RCC_Line
//=======================================================================

RCC_Line::RCC_Line ( const RCC_Object::location_t& lname, const RCC_Angle &dir ) : RCC_StatObject ( dir ), l_name ( lname )
{
}

RCC_Line::RCC_Line ( const RCC_Object::location_t& lname, const RCC_Vector2d &pos ) : RCC_StatObject ( pos ), l_name ( lname )
{
}

RCC_Line::~RCC_Line()
{
}

ostream& RCC_Line::print ( ostream& o, RCC_Vector2d::mode_t vm = RCC_Vector2d::POLAR, RCC_Angle::mode_t am = RCC_Angle::DEG ) const
{
  o << "((l " << RCC_Object::LOCATION_NAMES [ l_name ] << ") ";
  rel_pos.print ( o, vm, am );
  o << ")";
  return o;
}


//=======================================================================
//	RCC_MobObject
//=======================================================================

RCC_MobObject::RCC_MobObject ( const RCC_Angle& dir ) : RCC_Object ( dir )
{
}

RCC_MobObject::RCC_MobObject ( const RCC_Vector2d& pos ) : RCC_Object ( pos )
{
}

RCC_MobObject::RCC_MobObject ( const RCC_Vector2d& pos, const RCC_Vector2d& chg ) : RCC_Object ( pos ), rel_vel ( RCC_MobObject::vel ( pos, chg ) )
{
}


RCC_MobObject::~RCC_MobObject()
{
}

RCC_Vector2d RCC_MobObject::vel(const RCC_Vector2d &pos, const RCC_Vector2d &chg)
{
    if (!pos.isValid () || !chg.isValid () || 
	!chg.dir ().isValid () || chg.dist () == HUGE_VAL || 
	!pos.dir ().isValid () || pos.dist () == HUGE_VAL )
	return RCC_Vector2d();
   
    double dirC = chg.dir ().rad ();
    double objDist = pos.dist ();
    double distC = chg.dist ();

    RCC_Angle objDir( pos.dir () );
    
    RCC_Vector2d unitRCC_Vec = RCC_Vector2d ( 1.0, objDir );
 
    double x = distC * unitRCC_Vec.x () - dirC * objDist * unitRCC_Vec.y ();
    double y = distC * unitRCC_Vec.y () + dirC * objDist * unitRCC_Vec.x ();

    return RCC_Vector2d ( x, y );
}

RCC_Vector2d RCC_MobObject::distDirChg() const
{
    RCC_Vector2d unit = RCC_Vector2d ( 1.0, rel_pos.dir() );
    
    double distChg = rel_vel.x() * unit.x() + rel_vel.y() * unit.y();
    double dirChg = RCC_Angle( (rel_vel.y() * unit.x() - rel_vel.x() * unit.y() ), RCC_Angle::RAD ).deg () / rel_pos.dist();

    return RCC_Vector2d ( distChg, dirChg );
}


//=======================================================================
//	RCC_Flag
//=======================================================================


RCC_Flag::RCC_Flag ( const int &fname, const RCC_Angle &dir ) : RCC_MobObject ( dir ), f_name ( fname )
{
}

RCC_Flag::RCC_Flag ( const int &fname, const RCC_Vector2d &pos ) : RCC_MobObject ( pos ), f_name ( fname )
{
}

RCC_Flag::RCC_Flag ( const int &fname, const RCC_Vector2d &pos, const RCC_Vector2d &chg ) : RCC_MobObject ( pos, chg ), f_name ( fname )
{
}

RCC_Flag::~RCC_Flag()
{
}

ostream& RCC_Flag::print ( ostream& o, RCC_Vector2d::mode_t vm = RCC_Vector2d::POLAR, RCC_Angle::mode_t am = RCC_Angle::DEG ) const
{
  o << "((";
  if ( !isGoal () )
    o << "f ";

  o << getString () << ") ";
  rel_pos.print ( o, vm, am );
  if ( rel_vel.isValid () )
    {
      o << " ";
      distDirChg ().print ( o, RCC_Vector2d::CART, am );
    }
  o << ")";
  return o;
}

string RCC_Flag::getString () const
{
    if ( f_name < 0 )
	return "Unknown";

    string str;
    int fn1 = f_name % RCC_Object::MAX_LOC;
    
    str = RCC_Object::LOCATION_NAMES [ fn1 ];

    if ( f_name > RCC_Object::MAX_LOC )
	{
	    int f_name2 = ( f_name - fn1 ) / RCC_Object::MAX_LOC;
	    int fn2 = f_name2 % RCC_Object::MAX_LOC;
	    str += " " + RCC_Object::LOCATION_NAMES [ fn2 ];

	    if ( f_name2 > RCC_Object::MAX_LOC )
		{
		    int f_name3 = ( f_name2 - fn2 ) / RCC_Object::MAX_LOC;
		    if ( f_name3 < RCC_Object::MAX_LOC )
			str += " " + RCC_Object::LOCATION_NAMES [ f_name3 ];
		    else
		      {
			char tmp[32];
			strstream s ( tmp, 32 );
			s << f_name3;
			str += " ";
			str += tmp;
		      }
		}
	}
    else 
	{
	    if ( fn1 != RCC_Object::CENTER_LOC )
		str += " 0";
	}

    return str;
}

int RCC_Flag::getNum ()
{
    return RCC_Object::INVALID_LOC;
}

int RCC_Flag::getNum ( const RCC_Object::location_t& l)
{
    return l;
}

int RCC_Flag::getNum ( const RCC_Object::location_t& l1, const RCC_Object::location_t& l2 )
{
    return l1 + ( l2 * RCC_Object::MAX_LOC );
}

int RCC_Flag::getNum ( const RCC_Object::location_t& l1, const int&)
{
    return l1;
}

int RCC_Flag::getNum ( const RCC_Object::location_t& l1, const RCC_Object::location_t& l2, const int& l3 )
{
    return l1 + ( l2 * RCC_Object::MAX_LOC ) + ( l3 * RCC_Object::MAX_LOC * RCC_Object::MAX_LOC );
}

int RCC_Flag::getNum ( const RCC_Object::location_t& l1, const RCC_Object::location_t& l2, const location_t& l3 )
{
    return l1 + ( l2 * RCC_Object::MAX_LOC ) + ( l3 * RCC_Object::MAX_LOC * RCC_Object::MAX_LOC );
}



//=======================================================================
//	RCC_Ball
//=======================================================================

RCC_Ball::RCC_Ball ( const RCC_Angle & dir ) : RCC_MobObject ( dir )
{
}

RCC_Ball::RCC_Ball ( const RCC_Vector2d &pos ) : RCC_MobObject ( pos )
{
}

RCC_Ball::RCC_Ball ( const RCC_Vector2d &pos, const RCC_Vector2d &chg ) : RCC_MobObject ( pos, chg )
{
}

RCC_Ball::~RCC_Ball()
{
}

ostream& RCC_Ball::print ( ostream& o, RCC_Vector2d::mode_t vm = RCC_Vector2d::POLAR, RCC_Angle::mode_t am = RCC_Angle::DEG ) const
{
  o << "((b) ";
  rel_pos.print ( o, vm, am );
  if ( rel_vel.isValid () )
    {
      o << " ";
      distDirChg ().print ( o, RCC_Vector2d::CART, am );
    }
  o << ")";
  return o;
}

//=======================================================================
//	RCC_PlayerName
//=======================================================================

const int RCC_PlayerName::UNKNOWN_NUM = 0;
const string RCC_PlayerName::UNKNOWN_TEAM = "";

ostream& RCC_PlayerName::print ( ostream& o ) const
{
  if ( team_name != "" ) // then we wont know if they are a goalie either
    {
      o << "\"" << team_name << "\" " << unum;
      if ( goal_keeper )
	o << " goalie";
    }
  return o;
}

//=======================================================================
//	RCC_Player
//=======================================================================


RCC_Player::RCC_Player ( const RCC_PlayerName& pn, const RCC_Angle& dir) : RCC_MobObject ( dir ), p_name ( pn )
{
}

RCC_Player::RCC_Player ( const RCC_PlayerName& pn, const RCC_Vector2d &pos ) : RCC_MobObject ( pos ), p_name ( pn )
{
}

RCC_Player::RCC_Player ( const RCC_PlayerName& pn, const RCC_Vector2d &pos, const RCC_Vector2d &chg ) : RCC_MobObject ( pos, chg ), p_name ( pn )
{
}

RCC_Player::RCC_Player ( const RCC_PlayerName& pn, const RCC_Vector2d &pos, const RCC_Vector2d &chg, const RCC_Angle &body, const RCC_Angle &head ) 
    : RCC_MobObject ( pos, chg ), body_orientation ( body ), head_orientation ( head ), p_name ( pn )
{
}

RCC_Player::~RCC_Player()
{
}

ostream& RCC_Player::print ( ostream& o, RCC_Vector2d::mode_t vm = RCC_Vector2d::POLAR, RCC_Angle::mode_t am = RCC_Angle::DEG ) const
{
  o << "((p";
  if ( p_name.isKnown () )
    o << " " << p_name;
  o << ") ";

  rel_pos.print ( o, vm, am );
  if ( rel_vel.isValid () )
    {
      o << " ";
      distDirChg ().print ( o, RCC_Vector2d::CART, am ); 
    }
  if ( body_orientation.isValid () )
    {
      o << " ";
      body_orientation.print ( o, am );
    }
  if ( head_orientation.isValid () )
    {
      o << " ";
      head_orientation.print ( o, am );
    }

  o << ")";
  return o;
}
