#include "rccobjectpool.h"

RCC_ObjectPool::RCC_ObjectPool ( const string& tn ) : team_name ( tn ), op_time ( -1 )
{
}

RCC_ObjectPool::~RCC_ObjectPool ()
{
    
}

void RCC_ObjectPool::clear ()
{
  team_name = RCC_PlayerName::UNKNOWN_TEAM; 
  op_time = -1;
    
  lines.clear ();
  flags.clear ();
  allies.clear ();
  enemies.clear ();
  neuts.clear ();
  balls.clear ();
}

void RCC_ObjectPool::add ( const RCC_Object* obj )
{
  const RCC_Flag* flag = dynamic_cast <const RCC_Flag*> ( obj );
  if ( flag != NULL )
    {
      add ( *flag );
      return;
    }

  const RCC_Player* player = dynamic_cast <const RCC_Player*> ( obj );
  if ( player != NULL )
    {
      add ( *player );
      return;
    }
	
  const RCC_Ball* ball = dynamic_cast <const RCC_Ball*> ( obj );
  if ( ball != NULL )
    {
      add ( *ball );
      return;
    }
	    
  const RCC_Line* line = dynamic_cast <const RCC_Line*> ( obj );
  if ( line != NULL )
    {
      add ( *line );
      return;
    }

  cerr << "Could not cast from object\n"; 
} 

void RCC_ObjectPool::add ( const RCC_Line& line )
{
    lines.push_back ( line );
}

void RCC_ObjectPool::add ( const RCC_Flag& flag )
{
  if ( flag.isGoal () )
    goals.push_back ( flag );
  else
    flags.push_back ( flag );
}


void RCC_ObjectPool::add ( const RCC_Player& player )
{
  if ( team_name == RCC_PlayerName::UNKNOWN_TEAM || 
       player.playerName ().team () == RCC_PlayerName::UNKNOWN_TEAM )
    neuts.push_back ( player );
  
  else if ( player.playerName ().team() == team_name )
    allies.push_back ( player );
  
  else
    enemies.push_back ( player );
}

void RCC_ObjectPool::add ( const RCC_Ball& ball )
{
  balls.push_back ( ball );
} 




ostream& RCC_ObjectPool::print ( ostream& o, RCC_Vector2d::mode_t vm, RCC_Angle::mode_t am ) const
{
  o << "(see " << op_time;

  for ( unsigned int i = 0; i < goals.size(); i++ )
    {
      o << " ";
      goals [ i ].print ( o, vm, am );
    }

  for ( unsigned int i = 0; i < flags.size(); i++ )
    {
      o << " ";
      flags [ i ].print ( o, vm, am );
    }
    
  for ( unsigned int i = 0; i < allies.size(); i++ )
    {
      o << " ";
      allies [ i ].print ( o, vm, am );
    }

  for ( unsigned int i = 0; i < neuts.size(); i++ )
    {
      o << " ";
      neuts [ i ].print ( o, vm, am );
    }

  for ( unsigned int i = 0; i < enemies.size(); i++ )
    {
      o << " ";
      enemies [ i ].print ( o, vm, am );
    }

  for ( unsigned int i = 0; i < balls.size(); i++ )
    {
      o << " ";
      balls [ i ].print ( o, vm, am );
    }

  for ( unsigned int i = 0; i < lines.size(); i++ )
    {
      o << " ";
      lines [ i ].print ( o, vm, am ); 
    }
  
  o << ")";
  return o;
}
