#include "rccplaymode.h"

const string RCC_PlayMode::STAD_NAMES[] = PLAYMODE_STRINGS;
const string RCC_PlayMode::NAMES[] = 
{
  "Unknown",
  "BeforeKickOff",
  "TimeOver",
  "PlayOn",
  "KickOff",
  "KickIn",
  "FreeKick",
  "CornerKick",
  "GoalKick",
  "AfterGoal",
  "DropBall",
  "OffSide",
  "PK",
  "FirstHalfOver",
  "Pause",
  "Human",
  "FoulCharge",
  "FoulPush",
  "FoulMultipleAttacker",
  "FoulBallOut",
  "GoalieCatch",
  "TimeUpWithoutTeam",
  "TimeUp",
  "TimeExtended",
  "HalfTime",
  "Foul",
  "Goal"
};	

const string RCC_PlayMode::COACH_NAMES[] = 
{    
  "bko",
  "time_over",
  "play_on",
  "ko_our",		
  "ko_opp",		
  "ki_our",		
  "ki_opp",	
  "fk_our",
  "fk_opp",
  "ck_our",
  "ck_opp",
  "gk_our",
  "gk_opp",
  "gc_our",
  "gc_opp",
  "ag_our",
  "ag_opp"
};	

RCC_PlayMode::stad_mode_t RCC_PlayMode::stadMode ( const Side& team_side ) const
{
  switch ( mode_ )
    {
    case RCC_PlayMode::BeforeKickOff:
      return PM_BeforeKickOff;

    case RCC_PlayMode::TimeOver:
      return PM_TimeOver;

    case RCC_PlayMode::PlayOn:
      return PM_PlayOn;

    case RCC_PlayMode::KickOff:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_KickOff_Left;
	case RIGHT:
	  return PM_KickOff_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::KickIn:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_KickIn_Left;
	case RIGHT:
	  return PM_KickIn_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::FreeKick:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_FreeKick_Left;
	case RIGHT:
	  return PM_FreeKick_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::CornerKick:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_CornerKick_Left;
	case RIGHT:
	  return PM_CornerKick_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::GoalKick:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_GoalKick_Left;
	case RIGHT:
	  return PM_GoalKick_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::AfterGoal:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_AfterGoal_Left;
	case RIGHT:
	  return PM_AfterGoal_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::DropBall:
      return PM_Drop_Ball;

    case RCC_PlayMode::OffSide:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_OffSide_Left;
	case RIGHT:
	  return PM_OffSide_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::PK:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_PK_Left;
	case RIGHT:
	  return PM_PK_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::FirstHalfOver:
      return PM_FirstHalfOver;

    case RCC_PlayMode::Pause:
      return PM_Pause;

    case RCC_PlayMode::Human:
      return PM_Human;

    case RCC_PlayMode::FoulCharge:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_Foul_Charge_Left;
	case RIGHT:
	  return PM_Foul_Charge_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::FoulPush:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_Foul_Push_Left;
	case RIGHT:
	  return PM_Foul_Push_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::FoulMultipleAttacker:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_Foul_MultipleAttacker_Left;
	case RIGHT:
	  return PM_Foul_MultipleAttacker_Right;
	default:
	  return PM_Null;
	}

    case RCC_PlayMode::FoulBallOut:
      switch ( side_.stadMode ( team_side ) )
	{
	case LEFT:
	  return PM_Foul_BallOut_Left;
	case RIGHT:
	  return PM_Foul_BallOut_Right;
	default:
	  return PM_Null;
	}

    default:
      return PM_Null;
    }
}


RCC_PlayMode::coach_mode_t RCC_PlayMode::coachMode () const
{
   switch ( mode_ )
    {
    case RCC_PlayMode::BeforeKickOff:
      return RCC_PlayMode::bko;

    case RCC_PlayMode::TimeOver:
      return RCC_PlayMode::time_over;

    case RCC_PlayMode::PlayOn:
      return RCC_PlayMode::play_on;

 
    case RCC_PlayMode::KickOff:
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::ko_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::ko_opp;
	default:
	  return RCC_PlayMode::none;
	}

    case RCC_PlayMode::KickIn:
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::ki_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::ki_opp;
	default:
	  return RCC_PlayMode::none;
	}

    case RCC_PlayMode::FreeKick:
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::fk_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::fk_opp;
	default:
	  return RCC_PlayMode::none;
	}

    case RCC_PlayMode::CornerKick:
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::ck_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::ck_opp;
	default:
	  return RCC_PlayMode::none;
	}

    case RCC_PlayMode::GoalKick:
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::gk_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::gk_opp;
	default:
	  return RCC_PlayMode::none;
	}

    case RCC_PlayMode::AfterGoal:
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::ag_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::ag_opp;
	default:
	  return RCC_PlayMode::none;
	}

    case RCC_PlayMode::GoalieCatch: 
      switch ( side_.mode () )
	{
	case RCC_Side::Our:
	  return RCC_PlayMode::gc_our;
	case RCC_Side::Opp:
	  return RCC_PlayMode::gc_opp;
	default:
	  return RCC_PlayMode::none;
	}
    default:
      return RCC_PlayMode::none;
    }
}

RCC_PlayMode::mode_t RCC_PlayMode::mode ( const RCC_PlayMode::stad_mode_t& m, const Side& team_side )
{
    switch ( m )
    {
    case PM_Null:
    case PM_MAX:
    case PM_BeforeKickOff:
    case PM_TimeOver:
    case PM_PlayOn:
    case PM_FirstHalfOver:
    case PM_Pause:
    case PM_Human:
    case PM_Drop_Ball:
      side_.mode ( NEUTRAL, team_side );
      break;

    case PM_KickOff_Left:
    case PM_KickIn_Left:
    case PM_OffSide_Left:
    case PM_FreeKick_Left:
    case PM_PK_Left:
    case PM_Foul_Charge_Left:
    case PM_CornerKick_Left:
    case PM_Foul_Push_Left:
    case PM_GoalKick_Left:
    case PM_Foul_MultipleAttacker_Left:
    case PM_AfterGoal_Left:
    case PM_Foul_BallOut_Left:
      side_.mode ( LEFT, team_side );
      break;

    case PM_AfterGoal_Right:
    case PM_KickOff_Right:
    case PM_OffSide_Right:
    case PM_KickIn_Right:
    case PM_PK_Right:
    case PM_Foul_Charge_Right:
    case PM_FreeKick_Right:
    case PM_Foul_Push_Right:
    case PM_CornerKick_Right:
    case PM_Foul_MultipleAttacker_Right:
    case PM_GoalKick_Right:
    case PM_Foul_BallOut_Right:
      side_.mode ( RIGHT, team_side );
      break;
    }
    return mode ( m );
}

RCC_PlayMode::mode_t RCC_PlayMode::mode ( const RCC_PlayMode::stad_mode_t& m )
{
  switch ( m )
    {
    case PM_Null:
    case PM_MAX:
      return mode_ = RCC_PlayMode::Unknown;
    case PM_BeforeKickOff:
      return mode_ = RCC_PlayMode::BeforeKickOff;
    case PM_TimeOver:
      return mode_ = RCC_PlayMode::TimeOver;
    case PM_PlayOn:
      return mode_ = RCC_PlayMode::PlayOn;
    case PM_KickOff_Left:
    case PM_KickOff_Right:
      return mode_ = RCC_PlayMode::KickOff;
    case PM_KickIn_Left:
    case PM_KickIn_Right:
      return mode_ = RCC_PlayMode::KickIn;
    case PM_FreeKick_Left:
    case PM_FreeKick_Right:
      return mode_ = RCC_PlayMode::FreeKick;
    case PM_CornerKick_Left:
    case PM_CornerKick_Right:
      return mode_ = RCC_PlayMode::CornerKick;
    case PM_GoalKick_Left:
    case PM_GoalKick_Right:
      return mode_ = RCC_PlayMode::GoalKick;
    case PM_AfterGoal_Left:
    case PM_AfterGoal_Right:
      return mode_ = RCC_PlayMode::AfterGoal;
    case PM_Drop_Ball:
      return mode_ = RCC_PlayMode::DropBall;
    case PM_OffSide_Left:
    case PM_OffSide_Right:
      return mode_ = RCC_PlayMode::OffSide;
    case PM_PK_Left:
    case PM_PK_Right:
      return mode_ = RCC_PlayMode::PK;
    case PM_FirstHalfOver:
      return mode_ = RCC_PlayMode::FirstHalfOver;
    case PM_Pause:
      return mode_ = RCC_PlayMode::Pause;
    case PM_Human:
      return mode_ = RCC_PlayMode::Human;
    case PM_Foul_Charge_Left:
    case PM_Foul_Charge_Right:
      return mode_ = RCC_PlayMode::FoulCharge;
    case PM_Foul_Push_Left:
    case PM_Foul_Push_Right:
      return mode_ = RCC_PlayMode::FoulPush;
    case PM_Foul_MultipleAttacker_Left:
    case PM_Foul_MultipleAttacker_Right:
      return mode_ = RCC_PlayMode::FoulMultipleAttacker;
    case PM_Foul_BallOut_Left:
    case PM_Foul_BallOut_Right:
      return mode_ = RCC_PlayMode::FoulBallOut;
   }
}

RCC_PlayMode::mode_t RCC_PlayMode::mode ( const RCC_PlayMode::coach_mode_t& m )
{
  switch ( m )
    {
    case RCC_PlayMode::none:
      side_.mode ( RCC_Side::Neut );
      return mode_ = RCC_PlayMode::Unknown;
    case RCC_PlayMode::bko:
      side_.mode ( RCC_Side::Neut );
      return mode_ = RCC_PlayMode::BeforeKickOff;
    case RCC_PlayMode::time_over:
      side_.mode ( RCC_Side::Neut );
      return mode_ = RCC_PlayMode::TimeOver;
    case RCC_PlayMode::play_on:
      side_.mode ( RCC_Side::Neut );
      return mode_ = RCC_PlayMode::PlayOn;
    case RCC_PlayMode::ko_our:		
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::KickOff;
    case RCC_PlayMode::ko_opp:		
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::KickOff;
    case RCC_PlayMode::ki_our:		
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::KickIn;
    case RCC_PlayMode::ki_opp:	
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::KickIn;
    case RCC_PlayMode::fk_our:
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::FreeKick;
    case RCC_PlayMode::fk_opp:
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::FreeKick;
    case RCC_PlayMode::ck_our:
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::CornerKick;
    case RCC_PlayMode::ck_opp:
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::CornerKick;
    case RCC_PlayMode::gk_our:
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::GoalKick;
    case RCC_PlayMode::gk_opp:
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::GoalKick;
    case RCC_PlayMode::gc_our:
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::GoalieCatch;
    case RCC_PlayMode::gc_opp:
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::GoalieCatch;
    case RCC_PlayMode::ag_our:
      side_.mode ( RCC_Side::Our );
      return mode_ = RCC_PlayMode::AfterGoal;
    case RCC_PlayMode::ag_opp:
      side_.mode ( RCC_Side::Opp );
      return mode_ = RCC_PlayMode::AfterGoal;
    }
}


ostream& operator<< ( ostream& o, const PlayMode& pm )
{
  return o << RCC_PlayMode::STAD_NAMES [ pm ];
}

ostream& operator<< ( ostream& o, const RCC_PlayMode::coach_mode_t& pm )
{
  return o << RCC_PlayMode::COACH_NAMES [ pm ];
}
