/* -*- Mode: C++ -*- */
/*
 *Copyright:

    Copyright (C) 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Itsuki Noda,	Mikhail Prokopenko, Jan Wendler 

    This file is part of SoccerServer.

    SoccerServer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

/* This files defines a class region to specify regions for the coach messages */

#include "rccregion.h"

/*** RCC_RegQuad ***/

ostream& RCC_RegQuad::print ( ostream& out ) const 
{
  out << "(quad";
  for (int i=0; i<4; i++)
    out << " " << *pts[i];
  out << ")";
  return out;
}
  
ostream& RCC_RegQuad::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header
      << "Quadrangle: ";
  for ( int i = 0; i < 4; i++ ) 
    {
      pts[i]->printSimple ( out );
      out << " ";
    }
  out << endl;
  return out;
}



/*** RCC_RegArc ***/

ostream& RCC_RegArc::print ( ostream& out ) const
{
  out << "(arc"
      << " " << *center_p
      << " " << start_rad 
      << " " << end_rad 
      << " " << start_ang 
      << " " << span_ang
      << ")";
  return out;
}

ostream& RCC_RegArc::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header
      << "Arc: "
      << "center=";
  center_p->printSimple ( out );
  out << "\tradius="
      << start_rad
      << " to "
      << end_rad
      << "\tangle="
      << start_ang
      << " for "
      << span_ang
      << endl;

  return out;
}

void RCC_RegArc::rad ( const double& sr, const double& er )
{
  if ( sr > er )
    throw ( "Start radius greater than end radius" );
  start_rad = sr;
  end_rad = er;
}

void RCC_RegArc::ang ( const RCC_Angle& start, const RCC_Angle& span )
{
  start_ang = start;
  span_ang = span;
}

/*** RCC_RegUnion ***/
RCC_RegUnion::RCC_RegUnion ( const RCC_RegUnion& r )
{
  for ( RCC_Region::container::const_iterator i = r.regions_.begin(); i != r.regions_.end(); i++ )
    regions_.push_back ( ( *i )->copy () );
}


ostream& RCC_RegUnion::print ( ostream& out ) const
{
  out << "(reg";
  for (RCC_Region::container::const_iterator i = regions_.begin(); i != regions_.end(); i++ )
    out << " " << **i;
  out << ")";
  return out;  
}

ostream& RCC_RegUnion::printPretty ( ostream& out, const string& line_header ) const
{
  out << line_header << "RCC_Region Union:" << endl;
  for ( RCC_Region::container::const_iterator i = regions_.begin(); i != regions_.end(); i++ )
    ( *i )->printPretty ( out, line_header + "o " );
  return out;
}


RCC_Region* RCC_RegUnion::region ( const unsigned int& i )
{
  if ( i < regions_.size () )
    return regions_ [ i ];
  else
    throw ( "RCC_RegUnion idx out of bounds" ); 
}

void RCC_RegUnion::add ( const RCC_Region* r )
{
  regions_.push_back ( r->copy () );
}

void RCC_RegUnion::remove ( const unsigned int& i )
{
  unsigned int tmp = 0; 
  for ( RCC_Region::container::iterator j = regions_.begin(); j != regions_.end(); j++ )
    {
      if ( i == tmp )
	{
	regions_.erase ( j );
	delete *j;
	return;
	}
      tmp++;
    }
  throw ( "RCC_RegUnion idx out of bounds" );
}

void RCC_RegUnion::clear ()
{
  for ( RCC_Region::container::iterator i = regions_.begin(); i != regions_.end(); i++ )
    delete *i;
  regions_.clear ();
}





const string RCC_BallMoveSet::NAMES[] = { "NONE", "p", "d", "c", "s", "ALL" };
const unsigned int RCC_BallMoveSet::MAX = 4;

RCC_BallMoveSet::RCC_BallMoveSet () 
{ 
  clear(); 
}

RCC_BallMoveSet::RCC_BallMoveSet ( const RCC_BallMoveSet::RCC_BallMove_t& init_entry ) 
{ 
  clear(); 
  add ( init_entry ); 
}

RCC_BallMoveSet::~RCC_BallMoveSet () 
{
}


void RCC_BallMoveSet::add ( const RCC_BallMoveSet::RCC_BallMove_t& t )
{
  unsigned int i = ( unsigned int ) t;

  if ( i < 1 || i > RCC_BallMoveSet::MAX )
    throw ( "RCC_BallMoveSet ind out of bounds" ); 
 
  if ( t == RCC_BallMoveSet::All ) 
    entries = ~0;
  else 
    entries |= ( 1 << ( i - 1 ) );
}

void RCC_BallMoveSet::remove ( const RCC_BallMoveSet::RCC_BallMove_t& t )
{
  unsigned int i = ( unsigned int ) t;
  if ( i < 1 || i > RCC_BallMoveSet::MAX )
    throw ( "RCC_BallMoveSet idx out of bounds" ); 

  if ( t == RCC_BallMoveSet::All ) 
    clear();
  else 
    entries &= ~ ( 1 << ( i - 1 ) );
}

bool RCC_BallMoveSet::isMember ( const RCC_BallMoveSet::RCC_BallMove_t& t )
{
  unsigned int i = ( unsigned int ) t;

  if ( i < 1 || i > RCC_BallMoveSet::MAX )
    throw ( "RCC_BallMoveSet idx out of bounds" );

  if ( t == RCC_BallMoveSet::All ) 
    return ( ~( ( (~0) << ( RCC_BallMoveSet::MAX ) ) | entries ) ) == 0;
  else 
    return !!( entries & ( 1 << ( i - 1 ) ) );
}

ostream& RCC_BallMoveSet::print ( ostream& out ) const
{
  out << "{";
  for (int i = 1; i <= RCC_BallMoveSet::MAX; i++ )
    if ( entries & ( 1 << ( i - 1 ) ) ) 
      out << RCC_BallMoveSet::NAMES [ i ];
  out << "}";
  return out;
}

const unsigned int RCC_UnumSet::ALL = 0;

RCC_UnumSet::RCC_UnumSet ( int ts ) : team_size ( ts ) 
{ 
  clear(); 
}


RCC_UnumSet::~RCC_UnumSet () 
{
}


void RCC_UnumSet::add ( const unsigned int& i )
{
  if ( i > team_size )
    throw ( "RCC_UnumSet idx out of bounds" );
  if ( i == RCC_UnumSet::ALL ) 
    entries = ~0;
  else 
    entries |= ( 1 << ( i - 1 ) );
}

void RCC_UnumSet::remove ( const unsigned int& i )
{
  if ( i > team_size )
    throw ( "RCC_UnumSet idx out of bounds" );
  if ( i == RCC_UnumSet::ALL )
    clear();
  else
    entries &= ~( 1 << ( i - 1 ) );
}

bool RCC_UnumSet::isMember ( const unsigned int& i ) const
{
  if ( i > team_size)
    throw ( "RCC_UnumSet idx out of bounds" );
  if ( i == RCC_UnumSet::ALL )
    return ( ~( ( ( ~0 ) << team_size ) | entries ) ) == 0;
  else 
    return !!( entries & ( 1 << ( i - 1) ) );
}

ostream& RCC_UnumSet::print ( ostream& out ) const
{
  string space = "";
  out << "{";
  if ( isMember ( RCC_UnumSet::ALL ) )
    out << "0";
  else 
    {
      for ( int  i = 1; i <= team_size; i++)
	if ( entries & ( 1 << ( i - 1 ) ) ) 
	  {
	  out << space << i;
	  space = " ";
	  }
    }
  out << "}";
  return out;
}

