/* -*-flex-*- */
%{

#include "rccparser.h"
#include "rccparse.h"
  
#define YY_DECL int yylex ( void* lvalp, void* param )
#define LVALP ( ( YYSTYPE* ) lvalp )

#include <iostream.h>

#define SIDE ( ( ( rccparam_t* ) param )->side )
#define GC ( ( ( rccparam_t* ) param )->gc )

%}

%option nomain
%option noyywrap

DIGIT		[0-9]
UNSIGNED_INT	{DIGIT}+
INTEGER		-{0,1}{UNSIGNED_INT}
REAL 		{INTEGER}\.{UNSIGNED_INT}([eE]{INTEGER}){0,1}
%%

\(     		{ return LP; }
\) 		{ return RP; }

change_view 	{ return CHG_VIEW_KW; }
move 		{ return MOVE_KW; }
catch 		{ return CATCH_KW; }
turn_neck 	{ return TURN_NECK_KW; }
say 		{ return SAY_KW; }
turn 		{ return TURN_KW; }
dash 		{ return DASH_KW; }
kick 		{ return KICK_KW; }
view_mode 	{ return VIEW_MODE_KW; }
low 		{ return LOW_KW; }
high 		{ return HIGH_KW; }
narrow 		{ return NARROW_KW; }
normal 		{ return NORMAL_KW; }
wide 		{ return WIDE_KW; }
stamina 	{ return STAMINA_KW; }
head_angle 	{ return HEAD_ANGLE_KW; }
speed 		{ return SPEED_KW; }

sense_body 	{ return SENSE_BODY_KW; }
server_param    { return SERVER_PARAM_KW; }
player_param 	{ return PLAYER_PARAM_KW; }
player_type 	{ return PLAYER_TYPE_KW; }

see 		{ return SEE_KW; }
score		{ return SCORE_KW; }
goalie 		{ return GOALIE_KW; }

init 		{ return INIT_KW; }
reconnect 	{ return INIT_KW; }

hear 		{ return HEAR_KW; }
referee    	{ return REF_KW; }
coach			{ LVALP->side_p = new RCC_Side ( NEUTRAL, SIDE ); GC.add ( LVALP->side_p ); return COACH_KW; }
online_coach_right	{ LVALP->side_p = new RCC_Side ( RIGHT, SIDE ); GC.add ( LVALP->side_p ); return COACH_KW; }
online_coach_left	{ LVALP->side_p = new RCC_Side ( LEFT, SIDE ); GC.add ( LVALP->side_p ); return COACH_KW; } 

before_kick_off		 { LVALP->playmode_p = new RCC_PlayMode ( PM_BeforeKickOff, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
time_over		 { LVALP->playmode_p = new RCC_PlayMode ( PM_TimeOver, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
play_on			 { LVALP->playmode_p = new RCC_PlayMode ( PM_PlayOn, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
kick_off_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_KickOff_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
kick_off_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_KickOff_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
kick_in_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_KickIn_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
kick_in_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_KickIn_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
free_kick_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_FreeKick_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
free_kick_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_FreeKick_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
corner_kick_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_CornerKick_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
corner_kick_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_CornerKick_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goal_kick_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_GoalKick_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goal_kick_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_GoalKick_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goal_l			 { LVALP->playmode_p = new RCC_PlayMode ( PM_AfterGoal_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goal_r			 { LVALP->playmode_p = new RCC_PlayMode ( PM_AfterGoal_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
drop_ball	       	 { LVALP->playmode_p = new RCC_PlayMode ( PM_Drop_Ball, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
offside_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_OffSide_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
offside_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_OffSide_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
penalty_kick_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_PK_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
penalty_kick_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_PK_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
first_half_over		 { LVALP->playmode_p = new RCC_PlayMode ( PM_FirstHalfOver, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
pause			 { LVALP->playmode_p = new RCC_PlayMode ( PM_Pause, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
human_judge		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Human, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_charge_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_Charge_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_charge_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_Charge_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_push_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_Push_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_push_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_Push_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_multiple_attack_l	 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_MultipleAttacker_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_multiple_attack_r	 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_MultipleAttacker_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_ballout_l		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_BallOut_Left, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_ballout_r		 { LVALP->playmode_p = new RCC_PlayMode ( PM_Foul_BallOut_Right, SIDE );
                           GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

goalie_catch_ball_left	{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::GoalieCatch, RCC_Side ( LEFT, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goalie_catch_ball_right	{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::GoalieCatch, RCC_Side ( RIGHT, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
time_up_without_a_team	{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::TimeUpWithoutTeam, RCC_Side ( NEUTRAL, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
time_up			{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::TimeUp, RCC_Side ( NEUTRAL, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
time_extended		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::TimeExtended, RCC_Side ( NEUTRAL, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
half_time		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::HalfTime, RCC_Side ( NEUTRAL, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_left		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::Foul, RCC_Side ( LEFT, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
foul_right		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::Foul, RCC_Side ( RIGHT, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goal_left_		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::Goal, RCC_Side ( LEFT, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }
goal_right_		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::Goal, RCC_Side ( RIGHT, SIDE ) );
                          GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

bko		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::bko );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ko_our		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ko_our ); 
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ko_opp		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ko_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ki_our		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ki_our );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ki_opp		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ki_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

fk_our		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::fk_our );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

fk_opp		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::fk_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ck_our		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ck_our );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ck_opp		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ck_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

gk_our		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::gk_our );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

gk_opp		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::gk_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

gc_our    	{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::gc_our );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

gc_opp    	{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::gc_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ag_our		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ag_our );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

ag_opp		{ LVALP->playmode_p = new RCC_PlayMode ( RCC_PlayMode::ag_opp );
                  GC.add ( LVALP->playmode_p ); return PLAY_MODE_KW; }

meta 		{ return META;}
freeform 	{ return FREEFORM;}
info 		{ return INFO; }
advice 		{ return ADVICE; }
define  	{ return DEFINE; }
definec 	{ return DEFINEC; }
defined 	{ return DEFINED; }
definer 	{ return DEFINER; }
definea 	{ return DEFINEA; }

ver 		{ return VER;}

null 		{ return NULL_REGION;}
quad 		{ return QUAD;}
arc 		{ return ARC;}
pt 		{ return PT;}
reg 		{ return REG;}

pos 		{ return POS;}
home 		{ return HOME;}
bto 		{ return BALLTO;}
mark 		{ return MARK;}
markl 		{ return MARK_LINE;}
oline 		{ return OFFLINE;}
htype 		{ return HTYPE;}

do   		{ LVALP->bval = true; return DO_DONT; }
dont 		{ LVALP->bval = false; return DO_DONT; }

our  		{ LVALP->side_p = new RCC_Side ( RCC_Side::Our ); GC.add ( LVALP->side_p ); return TEAM; }
opp  		{ LVALP->side_p = new RCC_Side ( RCC_Side::Opp ); GC.add ( LVALP->side_p ); return TEAM; }

clear		{ return CLEAR; }

true		{ return TRUE_KW; }
false		{ return FALSE_KW; }
ppos		{ return PPOS; }
bpos		{ return BPOS; }
bowner		{ return BOWNER; }
and		{ return AND; }
or		{ return OR; }
not		{ return NOT; }
playm		{ return PLAY_M_KW; }

change_player_type { return CHANGE_KW; }

ball		{ return BALL_KW; }
[fF] 		{ return F_KW; }
t 		{ return T_KW; }
[bB] 		{ return B_KW; }
[lL] 		{ return L_KW; }
r 		{ return R_KW; }
c 		{ return C_KW; }
[pP] 		{ return P_KW; }
[gG] 		{ return G_KW; }
d 		{ return D_KW; }
s 		{ return S_KW; }

\"[0-9A-Za-z\(\)\.\+\-\*\/\?\<\>\_ ]{1,2046}\" { 
    int length = strlen ( yytext );
    strcpy ( LVALP->str, yytext + 1 ); /* skip quotation marks */
    LVALP->str[ length - 2 ] = 0;
    return QSTR; 
}


{REAL} 		{ LVALP->rval = atof ( yytext ); return RVAL; }
{INTEGER} 	{ LVALP->ival = atoi ( yytext ); return IVAL; }

([ \t])+ 	/* ignore white space */
\n		/* ignore newlines */

\{ 		{ return LB;}
\} 		{ return RB;}

.		{ fprintf (stderr, "Tokenizer Error: unmatched token - <%s>\n", yytext ); return ERROR; }

%%

int yyparse(void *);

extern "C"
{
  int RCC_Parse (const char* s, const string& team_name, const Side& side, RCC_SensorData& buffer )
  {
    rccparam_t params = { team_name, side, buffer };
    
    YY_BUFFER_STATE yybuffstateCurr = yy_scan_string(s);
    
    int ret = yyparse( (void*) &params );
    
    yy_delete_buffer(yybuffstateCurr);

    return ret;
  }
}	
