#include "rccvector2d.h"


RCC_Vector2d::RCC_Vector2d() : valid ( false )
{
}

RCC_Vector2d::RCC_Vector2d(const double& dist, const RCC_Angle& dir) 
  : valid ( true ), distance(dist), direction(dir), mode(POLAR)
{
}

RCC_Vector2d::RCC_Vector2d(const double& x, const double& y) 
  : valid ( true ), xval(x), yval(y), mode(CART)
{
}

RCC_Vector2d::~RCC_Vector2d()
{
}

RCC_Vector2d& RCC_Vector2d::operator+=(const RCC_Vector2d& i)
{
  if ( valid && i.valid )
    {
      try 
	{
	  toCart();
	  i.toCart();
 
	  xval += i.xval;
	  yval += i.yval;
	    
	  mode = CART;
	}
      catch ( ... )
	{
	  valid = false;
	}
   }
  else
    valid = false;

  return *this;
}

RCC_Vector2d& RCC_Vector2d::operator-=(const RCC_Vector2d& i)
{
  if ( valid && i.valid )
    {
      try 
	{
	  toCart();
	  i.toCart();

	  xval -= i.xval;
	  yval -= i.yval;
	    
	  mode = CART;
	}
      catch ( ... )
	{
	  valid = false;
	}
    }
  else
    valid = false;

  return *this;
}

RCC_Vector2d& RCC_Vector2d::operator*=(const double& scalar)
{
  if ( valid && scalar != HUGE_VAL )
    {
      switch ( mode )
	{
	case RCC_Vector2d::BOTH:
	  mode = RCC_Vector2d::POLAR;
	  // fall through
	case RCC_Vector2d::POLAR:
	  distance *= scalar;
	  break;

	case RCC_Vector2d::CART:
	  xval *= scalar;
	  yval *= scalar;
	  break;
	}
    }
  else
    valid = false;

  return *this;
}
    
RCC_Vector2d& RCC_Vector2d::operator/=(const double& scalar)
{
  if ( valid && scalar != 0.0 )
    {
      switch ( mode )
	{
	case RCC_Vector2d::BOTH:
	  mode = RCC_Vector2d::POLAR;
	  // fall through
	case RCC_Vector2d::POLAR:
	  distance /= scalar;
	  break;

	case RCC_Vector2d::CART:
	  xval /= scalar;
	  yval /= scalar;
	  break;
	}
    }
  else
    valid = false;

  return *this;
}

ostream& RCC_Vector2d::print ( ostream& o ) const
{ 
  return print ( o, RCC_Vector2d::POLAR, RCC_Angle::DEG ); 
}

ostream& RCC_Vector2d::print ( ostream& o, RCC_Vector2d::mode_t vm, RCC_Angle::mode_t am ) const
{
  if ( vm == RCC_Vector2d::POLAR ) 
    {
      toPolar ();
      if ( distance == HUGE_VAL )
	o << "unknown";
      else
	o << distance;

      o << " ";

      direction.print ( o, am );
    }
  else 
    {
      toCart ();
      if ( xval == HUGE_VAL )
	o << "unknown, ";
      else
	o << xval << " ";

     if ( yval == HUGE_VAL )
	o << "unknown";
      else
	o << yval;
    }

  return o;
}
  

bool equals(const RCC_Vector2d& a, const RCC_Vector2d& b)
{
  switch ( a.mode )
    {
    case RCC_Vector2d::POLAR:
      switch ( b.mode )
	{
	case RCC_Vector2d::CART:
	  try { b.toPolar (); } catch ( ... ) { return false; } 
	  // fall through
	case RCC_Vector2d::POLAR:
	case RCC_Vector2d::BOTH:
	  return ( a.distance != HUGE_VAL && b.distance != HUGE_VAL &&
		   a.direction.isValid () && b.direction.isValid () &&
		   a.distance == b.distance && a.direction == b.direction );
	}
   
    case RCC_Vector2d::CART:
      switch ( b.mode )
	{
	case RCC_Vector2d::POLAR:
	  try { a.toCart (); } catch ( ... ) { return false; } 
	  // fall through
	case RCC_Vector2d::CART:
	case RCC_Vector2d::BOTH:
	  return ( a.xval != HUGE_VAL && b.xval != HUGE_VAL &&
		   a.yval != HUGE_VAL && b.yval != HUGE_VAL &&
		   a.xval == b.xval && a.yval == b.yval );
	}

    case RCC_Vector2d::BOTH:
      switch ( b.mode )
	{
	case RCC_Vector2d::BOTH:
	case RCC_Vector2d::POLAR:
	  return ( a.distance != HUGE_VAL && b.distance != HUGE_VAL &&
		   a.direction.isValid () && b.direction.isValid () &&
		   a.distance == b.distance && a.direction == b.direction );
	  
	case RCC_Vector2d::CART:
	  return ( a.xval != HUGE_VAL && b.xval != HUGE_VAL &&
		   a.yval != HUGE_VAL && b.yval != HUGE_VAL &&
		   a.xval == b.xval && a.yval == b.yval );

	}
    }
}
